@@ -1,4 +1,5 @@
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  *  Header for DAAP (iTunes Music Sharing) hashing, connection
  *
  *  Copyright (C) 2004,2005 Charles Schmidt <cschmidt2@emich.edu>
@@ -43,13 +44,21 @@ typedef struct {
 #define RB_IS_DAAP_CONNECTION_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), RB_TYPE_DAAP_CONNECTION))
 #define RB_DAAP_CONNECTION_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), RB_TYPE_DAAP_CONNECTION, RBDAAPConnectionClass))
 
+typedef struct RBDAAPConnectionPrivate RBDAAPConnectionPrivate;
 
 typedef struct {
 	GObject parent;
+	RBDAAPConnectionPrivate *priv;
 } RBDAAPConnection;
 
 typedef struct {
 	GObjectClass parent;
+
+	char * (* authenticate) (RBDAAPConnection *connection,
+				 const char       *name);
+	void   (* connected   ) (RBDAAPConnection *connection);
+	void   (* disconnected) (RBDAAPConnection *connection);
+
 } RBDAAPConnectionClass;
 
 
