@@ -607,17 +607,25 @@ rb_uri_is_writable (const char *text_uri)
 gboolean
 rb_uri_is_local (const char *text_uri)
 {
-	gboolean ret = FALSE;
-	GnomeVFSURI *uri;
+	return g_str_has_prefix (text_uri, "file://");
+}
 
-	uri = gnome_vfs_uri_new (text_uri);
-	if (uri == NULL)
-		return FALSE;
+/*
+ * gnome_vfs_uri_new escapes a few extra characters that
+ * gnome_vfs_escape_path doesn't ('&' and '=').  If we
+ * don't adjust our URIs to match, we end up with duplicate
+ * entries, one with the characters encoded and one without.
+ */
+static char *
+escape_extra_gnome_vfs_chars (char *uri)
+{
+	if (strspn (uri, "&=") != strlen (uri)) {
+		char *tmp = gnome_vfs_escape_set (uri, "&=");
+		g_free (uri);
+		return tmp;
+	}
 
-	ret = gnome_vfs_uri_is_local (uri);
-	gnome_vfs_uri_unref (uri);
-
-	return ret;
+	return uri;
 }
 
 typedef struct {
@@ -646,6 +654,7 @@ rb_uri_handle_recursively_cb (const gchar *rel_path,
 	
 	if (info->type == GNOME_VFS_FILE_TYPE_REGULAR) {
 		char *escaped_rel_path = gnome_vfs_escape_path_string (rel_path);
+		escaped_rel_path = escape_extra_gnome_vfs_chars (escaped_rel_path);
 		path = g_build_filename (data->uri, escaped_rel_path, NULL);
 		(data->func) (path, data->user_data);
 		g_free (escaped_rel_path);
@@ -717,6 +726,7 @@ rb_canonicalise_uri (const char *uri)
 		result = gnome_vfs_make_path_name_canonical (uri);
 		tmp = gnome_vfs_escape_path_string (result);
 		g_free (result);
+		tmp = escape_extra_gnome_vfs_chars (tmp);
 		result = g_strconcat ("file://", tmp, NULL);
 		g_free (tmp);
 	} else  if (g_str_has_prefix (uri, "file://")) {
@@ -725,6 +735,7 @@ rb_canonicalise_uri (const char *uri)
 		tmp1  = gnome_vfs_unescape_string (uri + 7, NULL);  /* ignore "file://" */
 		tmp2 = gnome_vfs_escape_path_string (tmp1);
 		g_free (tmp1);
+		tmp2 = escape_extra_gnome_vfs_chars (tmp2);
 		result = g_strconcat ("file://", tmp2, NULL); /* re-add scheme */
 		g_free (tmp2);
 	} else {
@@ -742,3 +753,46 @@ rb_canonicalise_uri (const char *uri)
 	return result;
 }
 
+char*
+rb_uri_append_path (const char *uri, const char *path)
+{
+	GnomeVFSURI *vfs_uri, *full_uri;
+	char *result;
+	
+	vfs_uri = gnome_vfs_uri_new (uri);
+	full_uri = gnome_vfs_uri_append_path (vfs_uri, path);
+	gnome_vfs_uri_unref (vfs_uri);
+	result = gnome_vfs_uri_to_string (full_uri, GNOME_VFS_URI_HIDE_NONE);
+	gnome_vfs_uri_unref (full_uri);
+
+	return result;
+}
+
+char*
+rb_uri_append_uri (const char *uri, const char *fragment)
+{
+	GnomeVFSURI *vfs_uri, *full_uri;
+	char *result;
+	
+	vfs_uri = gnome_vfs_uri_new (uri);
+	full_uri = gnome_vfs_uri_append_string (vfs_uri, fragment);
+	gnome_vfs_uri_unref (vfs_uri);
+	result = gnome_vfs_uri_to_string (full_uri, GNOME_VFS_URI_HIDE_NONE);
+	gnome_vfs_uri_unref (full_uri);
+
+	return result;
+}
+
+char *
+rb_uri_get_dir_name (const char *uri)
+{
+	GnomeVFSURI *vfs_uri;
+	char *dirname;
+
+	vfs_uri = gnome_vfs_uri_new (uri);
+	dirname = gnome_vfs_uri_extract_dirname (vfs_uri);
+	gnome_vfs_uri_unref (vfs_uri);
+
+	return dirname;
+}
+
