@@ -30,10 +30,12 @@ extern const char RB_STOCK_SET_STAR[];
 extern const char RB_STOCK_UNSET_STAR[];
 extern const char RB_STOCK_NO_STAR[];
 extern const char RB_STOCK_PODCAST[];
+extern const char RB_STOCK_BROWSER[];
 extern const char GNOME_MEDIA_SHUFFLE[];
 extern const char GNOME_MEDIA_REPEAT[];
 extern const char GNOME_MEDIA_PLAYLIST[];
 extern const char GNOME_MEDIA_AUTO_PLAYLIST[];
+extern const char GNOME_MEDIA_EJECT[];
 
 void	rb_stock_icons_init	(void);
 
