@@ -40,6 +40,7 @@
 
 #include "rb-metadata.h"
 #include "rb-debug.h"
+#include "rb-util.h"
 #include "rb-file-helpers.h"
 
 G_DEFINE_TYPE(RBMetaData, rb_metadata, G_TYPE_OBJECT)
@@ -56,6 +57,7 @@ typedef GstElement *(*RBAddTaggerElem) (GstBin *, GstElement *);
 const char * ignore_mime_types[] = {
 	"image/",
 	"text/",
+	"application/xml"
 };
 
 struct RBMetadataGstType
@@ -73,6 +75,7 @@ struct RBMetaDataPrivate
 
 	GstElement *pipeline;
 	GstElement *sink;
+	gulong typefind_cb_id;
 
 	/* Array of RBMetadataGstType */
 	GPtrArray *supported_types;
@@ -137,67 +140,77 @@ rb_add_flac_tagger (GstBin *pipeline, GstElement *element)
 	return tagger;
 }
 
+
+#ifdef HAVE_GSTREAMER_0_10
+static void
+id3_pad_added_cb (GstElement *demux, GstPad *pad, GstElement *mux)
+{
+	GstPad *mux_pad;
+
+	mux_pad = gst_element_get_compatible_pad (mux, pad, NULL);
+	if (gst_pad_link (pad, mux_pad) != GST_PAD_LINK_OK)
+		rb_debug ("unable to link pad from id3demux to id3mux");
+	else
+		rb_debug ("linked pad from id3de to id3mux");
+}
+
 static GstElement *
 rb_add_id3_tagger (GstBin *pipeline, GstElement *element)
 {
-        GstElement *tagger = NULL;
-	GstElement *spider = NULL;
-	GstElement *identity = NULL;
-#ifdef HAVE_GSTREAMER_0_10
-	GstElement *filter = NULL;
-#endif
+	GstElement *demux = NULL;
+	GstElement *mux = NULL;
+
+	demux = gst_element_factory_make ("id3demux", NULL);
+	mux =  gst_element_factory_make ("id3mux", NULL);
+
+	if (demux == NULL || mux == NULL)
+		goto error;
+
+	gst_bin_add_many (pipeline, demux, mux, NULL);
+	if (!gst_element_link (element, demux))
+		goto error;
+
+	g_signal_connect (demux, "pad-added", (GCallback)id3_pad_added_cb, mux);
+
+	return mux;
+
+error:
+	g_object_unref (demux);
+	g_object_unref (mux);
+	return NULL;
+}
+#elif HAVE_GSTREAMER_0_8
+static GstElement *
+rb_add_id3_tagger (GstBin *pipeline, GstElement *element)
+{
+	GstElement *tagger, *identity;
 	GstCaps *filtercaps = NULL;
 
-        if (!(tagger = gst_element_factory_make("id3tag", "tagger"))) 
-                return NULL;
+	if (!(tagger = gst_element_factory_make ("id3tag", "tagger"))) 
+		return NULL;
 
-        if (!(identity = gst_element_factory_make("identity", "identity"))) {
-                gst_object_unref( GST_OBJECT(tagger) );
-                return NULL;
-        }
-	if (!(spider = gst_element_factory_make ("spider", "spider")) ||
-	    !(tagger = gst_element_factory_make ("id3mux", "id3mux")) ||
-	    !(filtercaps = gst_caps_new_simple ("application/x-id3", NULL))) {
-		gst_object_unref (GST_OBJECT (spider));
+	if (!(identity = gst_element_factory_make ("identity", "identity"))) {
 		gst_object_unref (GST_OBJECT (tagger));
-		gst_caps_unref (filtercaps);
 		return NULL;
 	}
 
-#ifdef HAVE_GSTREAMER_0_10
-	if (! (filter = gst_element_factory_make ("capsfilter", NULL))) {
-		gst_object_unref (GST_OBJECT (spider));
-		gst_object_unref (GST_OBJECT (tagger));
-		gst_caps_unref (filtercaps);
-		return NULL;
-	}
-	g_object_set (GST_OBJECT (tagger), "v1-tag", TRUE, NULL);
-#else
-	gst_element_set (tagger, "v1-tag", TRUE, NULL);
-#endif
-        
-        gst_bin_add_many (GST_BIN (pipeline), tagger, identity, NULL);
+	gst_bin_add_many (GST_BIN (pipeline), tagger, identity, NULL);
 
-#ifdef HAVE_GSTREAMER_0_8
+	filtercaps = gst_caps_new_simple ("application/x-id3", NULL);
 	if (!gst_element_link_filtered (tagger, identity, filtercaps)) {
-#endif
-#ifdef HAVE_GSTREAMER_0_10
-	g_object_set (G_OBJECT (filter), "filter-caps", filtercaps, NULL);
-	if (!gst_element_link_many (tagger, filter, identity, NULL)) {
-#endif
-
-		g_warning("Linking id3tag and identity failed");
-                gst_caps_unref(filtercaps);
-                gst_object_unref(GST_OBJECT(tagger));
-                gst_object_unref(GST_OBJECT(identity));
-                return NULL;
+		g_warning ("Linking id3tag and identity failed");
+		gst_caps_free (filtercaps);
+		gst_object_unref (GST_OBJECT(tagger));
+		gst_object_unref (GST_OBJECT(identity));
+		return NULL;
         }
-	gst_caps_unref (filtercaps);
+	gst_caps_free (filtercaps);
+
+	gst_element_link_many(element, tagger, NULL);
 
-        gst_element_link_many (element, tagger, NULL);
-        
 	return identity;
 }
+#endif
 
 static void
 add_supported_type (RBMetaData *md,
@@ -234,13 +247,20 @@ rb_metadata_init (RBMetaData *md)
  	 */
 	has_gnomevfssink = (gst_element_factory_find ("gnomevfssrc") != NULL &&
 			    gst_element_factory_find ("gnomevfssink") != NULL);
+#ifdef HAVE_GSTREAMER_0_10
+	has_id3 = (gst_default_registry_check_feature_version ("id3mux", 0, 10, 2) &&
+		   gst_element_factory_find ("id3demux") != NULL);
+#elif HAVE_GSTREAMER_0_8
 	has_id3 = (gst_element_factory_find ("id3tag") != NULL);
+#endif
 
 	tagger = (has_gnomevfssink && has_id3) ?  rb_add_id3_tagger : NULL;
 	add_supported_type (md, "application/x-id3", tagger, "MP3");
 	add_supported_type (md, "audio/mpeg", tagger, "MP3");
 
-	add_supported_type (md, "application/ogg", NULL, "Ogg");
+	add_supported_type (md, "application/ogg", NULL, "Ogg Vorbis");
+	add_supported_type (md, "audio/x-vorbis", NULL, "Ogg Vorbis");
+
 	add_supported_type (md, "audio/x-mod", NULL, "MOD");
 	add_supported_type (md, "audio/x-wav", NULL, "WAV");
 
@@ -450,7 +470,7 @@ rb_metadata_gst_load_tag (const GstTagList *list, const gchar *tag, RBMetaData *
 		return;
 	field = (RBMetaDataField) tem;
 
-	type = rb_metadata_get_field_type (md, field);
+	type = rb_metadata_get_field_type (field);
 	val = gst_tag_list_get_value_index (list, tag, 0);
 	newval = g_new0 (GValue, 1);
 	g_value_init (newval, type);
@@ -466,9 +486,19 @@ rb_metadata_gst_load_tag (const GstTagList *list, const gchar *tag, RBMetaData *
 
 	switch (type) {
 	case G_TYPE_STRING: {
-		/* Remove leading and trailing whitespace */
+		/* Reject invalid utf-8 strings, and then
+		 * remove leading and trailing whitespace.
+		 */ 
 		char *str;
 		str = g_value_dup_string (newval);
+
+		if (!g_utf8_validate (str, -1, NULL)) {
+			rb_debug ("Got invalid UTF-8 tag data");
+			g_free (str);
+			g_value_unset (newval);
+			g_free (newval);
+			return;
+		}
 		str = g_strstrip (str);
 		g_value_take_string (newval, str);
 		break;
@@ -558,10 +588,13 @@ rb_metadata_gst_eos_cb (GstElement *element, RBMetaData *md)
 static void
 rb_metadata_gst_typefind_cb (GstElement *typefind, guint probability, GstCaps *caps, RBMetaData *md)
 {
-	if (gst_caps_get_size (caps) > 0) {
+	if (!(gst_caps_is_empty (caps) || gst_caps_is_any (caps))) {
+		g_free (md->priv->type);
 		md->priv->type = g_strdup (gst_structure_get_name (gst_caps_get_structure (caps, 0)));
 		rb_debug ("found type %s", md->priv->type);
 	}
+
+	g_signal_handler_disconnect (typefind, md->priv->typefind_cb_id);
 }
 
 
@@ -626,8 +659,22 @@ static void
 rb_metadata_gst_unknown_type_cb (GstElement *decodebin, GstPad *pad, GstCaps *caps, RBMetaData *md)
 {
 	/* try to shortcut it a bit */
-	rb_debug ("decodebin emitted unknown type signal");
 	md->priv->non_audio = TRUE;
+
+	if (!gst_caps_is_empty (caps) && !gst_caps_is_any (caps)) {
+		GstStructure *structure;
+		const gchar *mimetype;
+
+		structure = gst_caps_get_structure (caps, 0);
+		mimetype = gst_structure_get_name (structure);
+
+		g_free (md->priv->type);
+		md->priv->type = g_strdup (mimetype);
+		
+		rb_debug ("decodebin emitted unknown type signal for %s", mimetype);
+	} else {
+		rb_debug ("decodebin emitted unknown type signal");
+	}
 }
 
 static GstElement *make_pipeline_element (GstElement *pipeline, const char *element, GError **error)
@@ -813,7 +860,11 @@ rb_metadata_load (RBMetaData *md,
  	 */
  	typefind = gst_bin_get_by_name (GST_BIN (decodebin), "typefind");
  	g_assert (typefind != NULL);
- 	g_signal_connect_object (typefind, "have_type", G_CALLBACK (rb_metadata_gst_typefind_cb), md, 0);
+	md->priv->typefind_cb_id = g_signal_connect_object (typefind, 
+							    "have_type", 
+							    G_CALLBACK (rb_metadata_gst_typefind_cb), 
+							    md, 
+							    0);
  
  	gst_element_link (urisrc, decodebin);
 
@@ -842,9 +893,10 @@ rb_metadata_load (RBMetaData *md,
 			    &state, NULL, 1 * GST_SECOND);
 	    change_timeout--;
 	}
-	if (state_ret == GST_STATE_CHANGE_ASYNC) {
+	if (state_ret != GST_STATE_CHANGE_SUCCESS) {
 		rb_debug ("failed to go to PAUSED for %s", uri);
-		if (!md->priv->non_audio)
+		rb_metadata_event_loop (md, GST_ELEMENT (pipeline), FALSE);
+		if (!md->priv->non_audio && md->priv->error == NULL)
 			g_set_error (error,
 				     RB_METADATA_ERROR,
 				     RB_METADATA_ERROR_INTERNAL,
@@ -852,7 +904,7 @@ rb_metadata_load (RBMetaData *md,
 	} else
 		rb_debug ("gone to PAUSED for %s", uri);
 
-	if (state_ret != GST_STATE_CHANGE_FAILURE && state_ret != GST_STATE_CHANGE_ASYNC) {
+	if (state_ret == GST_STATE_CHANGE_SUCCESS) {
 		/* Post application specific message so we'll know when to stop
 		 * the message loop */
 		GstBus *bus;
@@ -912,7 +964,7 @@ rb_metadata_load (RBMetaData *md,
 #endif
 
 	/* report errors for various failure cases.
-	 * these don't include the URI as the load failure dialog
+	 * these don't include the URI as the import errors source
 	 * already displays it.
 	 */
 	if (md->priv->error) {
@@ -937,16 +989,31 @@ rb_metadata_load (RBMetaData *md,
 			}
 		}
 
-		if (!ignore)
-			g_set_error (error,
-				     RB_METADATA_ERROR,
-				     RB_METADATA_ERROR_NOT_AUDIO,
-				     _("The file is not an audio stream"));
-		else
+
+		if (!ignore) {
+			const char *human_name;
+
+			human_name= rb_metadata_gst_type_to_name (md, md->priv->type);
+			if (human_name == NULL)
+				human_name = rb_mime_get_friendly_name (md->priv->type);
+			
+			if (human_name)
+				g_set_error (error,
+					     RB_METADATA_ERROR,
+					     RB_METADATA_ERROR_NOT_AUDIO,
+					     _("The GStreamer plugins to decode \"%s\" files cannot be found"),
+					     human_name);
+			else
+				g_set_error (error,
+					     RB_METADATA_ERROR,
+					     RB_METADATA_ERROR_NOT_AUDIO,
+					     _("The file contains a stream of type %s, which is not decodable"),
+					     md->priv->type);
+		} else
 			g_set_error (error,
 				     RB_METADATA_ERROR,
 				     RB_METADATA_ERROR_NOT_AUDIO_IGNORE,
-				     _("The file is not an audio stream, but is being ignored"));
+				     NULL);
 	} else if (!md->priv->handoff) {
 		const char *report_type = rb_metadata_gst_type_to_name (md, md->priv->type);
 		if (report_type == NULL)
@@ -955,7 +1022,7 @@ rb_metadata_load (RBMetaData *md,
 			     RB_METADATA_ERROR,
 			     RB_METADATA_ERROR_UNSUPPORTED,
 			     _("There is no plugin installed to handle a %s file."),
-			     md->priv->type);
+			     report_type);
 	} else {
 		/* yay, it worked */
 		rb_debug ("successfully read metadata for %s", uri);
@@ -997,23 +1064,23 @@ rb_metadata_gst_add_tag_data (gpointer key, const GValue *val, GstTagSetter *tag
 		return;
 
 	if (tag) {
-		GValue newval = {0,};
+		if (field == RB_METADATA_FIELD_DATE && g_value_get_ulong (val) == 0) {
+			/* we should ask gstreamer to remove the tag,
+			 * but there is no easy way of doing so
+			 */
+		} else {
+			GValue newval = {0,};
 
-		g_value_init (&newval, gst_tag_get_type (tag));
-		if (g_value_transform (val, &newval)) {
-			rb_debug("Setting %s",tag);
+			g_value_init (&newval, gst_tag_get_type (tag));
+			if (g_value_transform (val, &newval)) {
+				rb_debug("Setting %s",tag);
 
-			if (field == RB_METADATA_FIELD_DATE && g_value_get_ulong (val) == 0) {
-				/* we should ask gstreamer to remove the tag,
-				 * but there is no easy way of doing so
-				 */
-			} else {
 				gst_tag_setter_add_tag_values (GST_TAG_SETTER (tagsetter),
-							   GST_TAG_MERGE_REPLACE,
+							   GST_TAG_MERGE_REPLACE_ALL,
 							   tag, &newval, NULL);
 			}
+			g_value_unset (&newval);
 		}
-		g_value_unset (&newval);
 	}
 }
 
@@ -1108,7 +1175,7 @@ rb_metadata_save (RBMetaData *md, GError **error)
 		goto out_error;
 	}
 
-	gst_tag_setter_set_tag_merge_mode (GST_TAG_SETTER (tagger), GST_TAG_MERGE_REPLACE);
+	gst_tag_setter_set_tag_merge_mode (GST_TAG_SETTER (tagger), GST_TAG_MERGE_REPLACE_ALL);
 	g_hash_table_foreach (md->priv->metadata, 
 			      (GHFunc) rb_metadata_gst_add_tag_data,
 			      GST_TAG_SETTER (tagger));
@@ -1216,7 +1283,7 @@ rb_metadata_set (RBMetaData *md, RBMetaDataField field,
 		return FALSE;
 	}
 
-	type = rb_metadata_get_field_type (md, field);
+	type = rb_metadata_get_field_type (field);
 	g_return_val_if_fail (type == G_VALUE_TYPE (val), FALSE);
 
 	newval = g_new0 (GValue, 1);
