@@ -1,4 +1,5 @@
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  *  arch-tag: Implementation of Rhythmbox Bonobo remoting
  *
  *  Copyright (C) 2004 Colin Walters <walters@gnome.org>
@@ -19,6 +20,15 @@
  *
  */
 
+#include <config.h>
+
+#include <string.h>
+
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
+#include <gdk/gdk.h>
+#include <libgnome/libgnome.h>
+
 #include "rb-remote-bonobo.h"
 #include "rb-remote-client-proxy.h"
 #include <Rhythmbox.h>
@@ -29,12 +39,6 @@
 #include <bonobo/bonobo-window.h>
 #include <bonobo/bonobo-control-frame.h>
 #include <bonobo-activation/bonobo-activation-register.h>
-#include <gtk/gtk.h>
-#include <gdk/gdk.h>
-#include <config.h>
-#include <string.h>
-#include <libgnome/libgnome.h>
-#include <libgnome/gnome-i18n.h>
 
 #include "rb-debug.h"
 
@@ -146,6 +150,8 @@ struct RBRemoteBonoboPrivate
 	GParamSpec *property_spec[16];
 };
 
+#define RB_REMOTE_BONOBO_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_REMOTE_BONOBO, RBRemoteBonoboPrivate))
+
 GType
 rb_remote_bonobo_get_type (void)
 {
@@ -214,6 +220,8 @@ rb_remote_bonobo_class_init (RBRemoteBonoboClass *klass)
 	epv->setRating = rb_remote_bonobo_corba_set_rating;
 	epv->skip = rb_remote_bonobo_corba_skip;
 	epv->toggleMute = rb_remote_bonobo_corba_toggle_mute;
+
+	g_type_class_add_private (klass, sizeof (RBRemoteBonoboPrivate));
 }
 
 static void
@@ -246,10 +254,67 @@ rb_remote_bonobo_remote_client_proxy_init (RBRemoteClientProxyIface *iface)
 	iface->get_visibility = rb_remote_bonobo_client_get_visibility_impl;
 }
 
+/* hack to use a recursive mutex for gdk locking */
+static GStaticRecMutex _rb_bonobo_mutex = G_STATIC_REC_MUTEX_INIT;
+static gboolean set_lock_functions = FALSE;
+
+static void
+rb_bonobo_workaround_lock (void)
+{
+	g_static_rec_mutex_lock (&_rb_bonobo_mutex);
+}
+
+static void
+rb_bonobo_workaround_unlock (void)
+{
+	g_static_rec_mutex_unlock (&_rb_bonobo_mutex);
+}
+
+void
+rb_remote_bonobo_preinit (void)
+{
+
+	/*
+	 * By default, GDK_THREADS_ENTER/GDK_THREADS_LEAVE uses a 
+	 * non-recursive mutex; this leads to deadlock, as there are
+	 * many code paths that lead to (for example) gconf operations
+	 * with the gdk lock held.  While performing these gconf operations,
+	 * ORBit will process incoming bonobo remote interface requests.
+	 * The implementations of the bonobo request handlers attempt
+	 * to acquire the gdk lock (as far as I know this is necessary, as
+	 * some operations will result in UI updates etc.); if the mutex
+	 * does not support recursive locks, this will deadlock.
+	 *
+	 * Dropping the gdk lock before all code paths that will possibly
+	 * lead to a gconf operation is way too hard (they're *everywhere*),
+	 * and unless someone can find a way of implementing the entire
+	 * remote interface without needing to acquire the gdk lock, this
+	 * is what we're stuck with.
+	 *
+	 ***
+	 *
+	 * Note: this is why CORBA sucks.  Arbitrary reentrancy is
+	 * nearly impossible to get right in an application with
+	 * significant global state (as nearly every GUI app has). The
+	 * D-BUS approach of queueing requests may lead to deadlocks,
+	 * but it's very obvious when this happens, and it's a lot
+	 * easier to debug and fix.  The above approach of making the
+	 * GDK lock recursive only partially helps; I am certain
+	 * there are code paths in Rhythmbox which are not expecting
+	 * to be reentered, particularly in RBShellPlayer.
+	 */
+	if (!set_lock_functions)
+	  {
+	    gdk_threads_set_lock_functions (G_CALLBACK (rb_bonobo_workaround_lock),
+					    G_CALLBACK (rb_bonobo_workaround_unlock));
+	    set_lock_functions = TRUE;
+	  }
+}
+
 static void
 rb_remote_bonobo_init (RBRemoteBonobo *bonobo) 
 {
-	bonobo->priv = g_new0 (RBRemoteBonoboPrivate, 1);
+	bonobo->priv = RB_REMOTE_BONOBO_GET_PRIVATE (bonobo);
 }
 
 static void
@@ -265,10 +330,6 @@ rb_remote_bonobo_dispose (GObject *object)
 static void
 rb_remote_bonobo_finalize (GObject *object)
 {
-        RBRemoteBonobo *bonobo = RB_REMOTE_BONOBO (object);
-
-	g_free (bonobo->priv);
-
 	(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
