@@ -24,11 +24,12 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
-
 #include <glib/gi18n.h>
+
 #include "rhythmdb-property-model.h"
 #include "rb-debug.h"
 #include "gsequence.h"
+#include "rb-refstring.h"
 #include "rb-tree-dnd.h"
 
 static void rhythmdb_property_model_tree_model_init (GtkTreeModelIface *iface);
@@ -40,6 +41,13 @@ G_DEFINE_TYPE_WITH_CODE(RhythmDBPropertyModel, rhythmdb_property_model, G_TYPE_O
 			G_IMPLEMENT_INTERFACE(RB_TYPE_TREE_DRAG_SOURCE,
 					      rhythmdb_property_model_drag_source_init))
 
+typedef struct {
+	RBRefString *string;
+	RBRefString *sort_string;
+	guint refcount;
+} RhythmDBPropertyModelEntry;
+
+
 static void rhythmdb_property_model_finalize (GObject *object);
 static void rhythmdb_property_model_set_property (GObject *object,
 					       guint prop_id,
@@ -61,7 +69,8 @@ static void rhythmdb_property_model_prop_changed_cb (RhythmDB *db, RhythmDBEntry
 static void rhythmdb_property_model_entry_removed_cb (RhythmDBQueryModel *model,
 						      RhythmDBEntry *entry,
 						      RhythmDBPropertyModel *propmodel);
-static void rhythmdb_property_model_insert (RhythmDBPropertyModel *model, RhythmDBEntry *entry);
+static RhythmDBPropertyModelEntry* rhythmdb_property_model_insert (RhythmDBPropertyModel *model,
+								   RhythmDBEntry *entry);
 static void rhythmdb_property_model_delete (RhythmDBPropertyModel *model,
 					    RhythmDBEntry *entry);
 static void rhythmdb_property_model_delete_prop (RhythmDBPropertyModel *model,
@@ -98,6 +107,7 @@ static gboolean rhythmdb_property_model_drag_data_delete (RbTreeDragSource *drag
 							  GList *paths);
 static gboolean rhythmdb_property_model_row_draggable (RbTreeDragSource *dragsource,
 						       GList *paths);
+
 enum {
 	TARGET_ALBUMS,
 	TARGET_GENRE,
@@ -127,18 +137,14 @@ static const GtkTargetEntry targets_location [] = {
 static GtkTargetList *rhythmdb_property_model_album_drag_target_list = NULL;
 static GtkTargetList *rhythmdb_property_model_artist_drag_target_list = NULL;
 static GtkTargetList *rhythmdb_property_model_genre_drag_target_list = NULL;
-
-typedef struct {
-	RBRefString *string;
-	RBRefString *sort_string;
-	guint refcount;
-} RhythmDBPropertyModelEntry;
+static GtkTargetList *rhythmdb_property_model_location_drag_target_list = NULL;
 
 struct RhythmDBPropertyModelPrivate
 {
 	RhythmDB *db;
 
 	RhythmDBQueryModel *query_model;
+	GHashTable *entries;
 
 	RhythmDBPropType propid;
 	RhythmDBPropType sort_propid;
@@ -168,7 +174,6 @@ enum
 {
 	PROP_0,
 	PROP_RHYTHMDB,
-	PROP_QUERY,
 	PROP_PROP,
 	PROP_QUERY_MODEL,
 };
@@ -192,6 +197,10 @@ rhythmdb_property_model_class_init (RhythmDBPropertyModelClass *klass)
 		rhythmdb_property_model_genre_drag_target_list =
 			gtk_target_list_new (targets_genre,
 					     G_N_ELEMENTS (targets_genre));
+	if (!rhythmdb_property_model_location_drag_target_list)
+		rhythmdb_property_model_location_drag_target_list =
+			gtk_target_list_new (targets_location,
+					     G_N_ELEMENTS (targets_location));
 
 	object_class->set_property = rhythmdb_property_model_set_property;
 	object_class->get_property = rhythmdb_property_model_get_property;
@@ -216,13 +225,6 @@ rhythmdb_property_model_class_init (RhythmDBPropertyModelClass *klass)
 							      RHYTHMDB_TYPE,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
-	g_object_class_install_property (object_class,
-					 PROP_QUERY,
-					 g_param_spec_pointer ("query",
-							      "Query",
-							      "RhythmDBQuery",
-							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-
 	g_object_class_install_property (object_class,
 					 PROP_PROP,
 					 g_param_spec_int ("prop",
@@ -267,11 +269,30 @@ rhythmdb_property_model_drag_source_init (RbTreeDragSourceIface *iface)
 	iface->drag_data_get = rhythmdb_property_model_drag_data_get;
 }
 
+static gboolean
+_remove_entry_cb (RhythmDBEntry *entry,
+		  gpointer unused,
+		  RhythmDBPropertyModel *model)
+{
+	rhythmdb_property_model_delete (model, entry);
+	return TRUE;
+}
+
+static gboolean
+_add_entry_cb (GtkTreeModel *model,
+	       GtkTreePath *path,
+	       GtkTreeIter *iter,
+	       RhythmDBPropertyModel *propmodel)
+{
+	rhythmdb_property_model_row_inserted_cb (model, path, iter, propmodel);
+	return FALSE;
+}
+
 static void
 rhythmdb_property_model_set_property (GObject *object,
-				   guint prop_id,
-				   const GValue *value,
-				   GParamSpec *pspec)
+				      guint prop_id,
+				      const GValue *value,
+				      GParamSpec *pspec)
 {
 	RhythmDBPropertyModel *model = RHYTHMDB_PROPERTY_MODEL (object);
 
@@ -281,11 +302,6 @@ rhythmdb_property_model_set_property (GObject *object,
 		model->priv->db = g_value_get_object (value);
 		break;
 	}
-	case PROP_QUERY:
-		if (model->priv->query)
-			rhythmdb_query_free (model->priv->query);
-		model->priv->query = rhythmdb_query_copy (g_value_get_pointer (value));
-		break;
 	case PROP_PROP:
 		model->priv->propid = g_value_get_int (value);
 		switch (model->priv->propid) {
@@ -322,12 +338,15 @@ rhythmdb_property_model_set_property (GObject *object,
 			g_signal_handlers_disconnect_by_func (G_OBJECT (model->priv->query_model),
 							      G_CALLBACK (rhythmdb_property_model_prop_changed_cb),
 							      model);
+			g_hash_table_foreach_remove (model->priv->entries, (GHRFunc)_remove_entry_cb, model);
 			g_object_unref (G_OBJECT (model->priv->query_model));
 		}
 		
 		model->priv->query_model = g_value_get_object (value);
+		g_assert (rhythmdb_property_model_iter_n_children (GTK_TREE_MODEL (model), NULL) == 1);
 
 		if (model->priv->query_model) {
+			g_object_ref (G_OBJECT (model->priv->query_model));
 			g_signal_connect_object (G_OBJECT (model->priv->query_model),
 						 "row_inserted",
 						 G_CALLBACK (rhythmdb_property_model_row_inserted_cb),
@@ -343,7 +362,9 @@ rhythmdb_property_model_set_property (GObject *object,
 						 G_CALLBACK (rhythmdb_property_model_prop_changed_cb),
 						 model,
 						 0);
-			g_object_ref (G_OBJECT (model->priv->query_model));
+			gtk_tree_model_foreach (GTK_TREE_MODEL (model->priv->query_model),
+						(GtkTreeModelForeachFunc)_add_entry_cb,
+						model);
 		}
 		break;
 	}
@@ -365,9 +386,6 @@ rhythmdb_property_model_get_property (GObject *object,
 	case PROP_RHYTHMDB:
 		g_value_set_object (value, model->priv->db);
 		break;
-	case PROP_QUERY:
-		g_value_set_pointer (value, model->priv->query);
-		break;
 	case PROP_PROP:
 		g_value_set_int (value, model->priv->propid);
 		break;
@@ -389,6 +407,7 @@ rhythmdb_property_model_init (RhythmDBPropertyModel *model)
 
 	model->priv->properties = g_sequence_new (NULL);
 	model->priv->reverse_map = g_hash_table_new (g_str_hash, g_str_equal);
+	model->priv->entries = g_hash_table_new (g_direct_hash, g_direct_equal);
 
 	model->priv->all = g_new0 (RhythmDBPropertyModelEntry, 1);
 	model->priv->all->string = rb_refstring_new (_("All"));
@@ -423,6 +442,7 @@ rhythmdb_property_model_finalize (GObject *object)
 
 	g_hash_table_destroy (model->priv->reverse_map);
 	g_sequence_free (model->priv->properties);
+	g_hash_table_destroy (model->priv->entries);
 
 	if (model->priv->query_model)
 		g_object_unref (G_OBJECT (model->priv->query_model));
@@ -436,59 +456,67 @@ rhythmdb_property_model_new (RhythmDB *db, RhythmDBPropType propid)
 	return g_object_new (RHYTHMDB_TYPE_PROPERTY_MODEL, "db", db, "prop", propid, NULL);
 }
 
-static inline RhythmDBEntry *
-entry_from_tree_iter (GtkTreeModel *model, GtkTreeIter *iter)
-{
-	RhythmDBEntry *entry;
-	gtk_tree_model_get (model, iter, 0, &entry, -1);
-	return entry;
-}
-
 static void
 rhythmdb_property_model_row_inserted_cb (GtkTreeModel *model,
 					 GtkTreePath *path,
 					 GtkTreeIter *iter,
 					 RhythmDBPropertyModel *propmodel)
 {
-	RhythmDBEntry *entry = entry_from_tree_iter (model, iter);
+	RhythmDBEntry *entry = rhythmdb_query_model_iter_to_entry (RHYTHMDB_QUERY_MODEL (model), iter);
+	RhythmDBPropertyModelEntry *prop;
 
-	if (rhythmdb_entry_get_boolean (entry, RHYTHMDB_PROP_HIDDEN) == FALSE) {
-		rhythmdb_property_model_insert (propmodel, entry);
-		rhythmdb_property_model_sync (propmodel);
-	}
+	if (g_hash_table_lookup (propmodel->priv->entries, entry))
+		return;
+
+
+	prop = rhythmdb_property_model_insert (propmodel, entry);
+	g_hash_table_insert (propmodel->priv->entries, entry, prop);
+
+	rhythmdb_property_model_sync (propmodel);
 }
 
 static void
 rhythmdb_property_model_prop_changed_cb (RhythmDB *db, RhythmDBEntry *entry,
-					 RhythmDBPropType prop, const GValue *old,
+					 RhythmDBPropType propid, const GValue *old,
 					 const GValue *new,
 					 RhythmDBPropertyModel *propmodel)
 {
-	if (prop == RHYTHMDB_PROP_HIDDEN) {
-		gboolean old_hidden = g_value_get_boolean (old);
-		gboolean new_hidden = g_value_get_boolean (new);
+	if (propid == RHYTHMDB_PROP_HIDDEN) {
+		gboolean old_val = g_value_get_boolean (old);
+		gboolean new_val = g_value_get_boolean (new);
 
-		if (new_hidden == old_hidden)
+		if (old_val != new_val) {
+			if (new_val == FALSE) {
+				RhythmDBPropertyModelEntry *prop;
+
+				if (g_hash_table_lookup (propmodel->priv->entries, entry) != NULL)
+					return;
+
+				prop = rhythmdb_property_model_insert (propmodel, entry);
+				g_hash_table_insert (propmodel->priv->entries, entry, prop);
+			} else {
+				if (g_hash_table_lookup (propmodel->priv->entries, entry) == NULL)
+					return;
+
+				rhythmdb_property_model_delete (propmodel, entry);
+				g_hash_table_remove (propmodel->priv->entries, entry);
+			}
+		}
+	} else {
+		RhythmDBPropertyModelEntry *prop;
+		
+		if (propid != propmodel->priv->propid)
 			return;
 
-		rb_debug ("recieving hidden-change notification for %s", entry->location);
-
-		if (new_hidden)
-			rhythmdb_property_model_delete (propmodel, entry);
-		else
-			rhythmdb_property_model_insert (propmodel, entry);
-
-		rhythmdb_property_model_sync (propmodel);
-	} else {	
-		if (prop != propmodel->priv->propid)
-			return;
-		if (rhythmdb_entry_get_boolean (entry, RHYTHMDB_PROP_HIDDEN))
+		if (g_hash_table_lookup (propmodel->priv->entries, entry) == NULL)
 			return;
 
-		rhythmdb_property_model_delete_prop (propmodel, g_value_get_string (old));
-		rhythmdb_property_model_insert (propmodel, entry);
-		rhythmdb_property_model_sync (propmodel);
+		rhythmdb_property_model_delete (propmodel, entry);
+		prop = rhythmdb_property_model_insert (propmodel, entry);
+		g_hash_table_insert (propmodel->priv->entries, entry, prop);
 	}
+
+	rhythmdb_property_model_sync (propmodel);
 }
 
 static void
@@ -496,10 +524,12 @@ rhythmdb_property_model_entry_removed_cb (RhythmDBQueryModel *model,
 					  RhythmDBEntry *entry,
 					  RhythmDBPropertyModel *propmodel)
 {
-	if (rhythmdb_entry_get_boolean (entry, RHYTHMDB_PROP_HIDDEN) == FALSE) {
-		rhythmdb_property_model_delete (propmodel, entry);
-		rhythmdb_property_model_sync (propmodel);
-	}
+	if (g_hash_table_lookup (propmodel->priv->entries, entry) == NULL)
+		return;
+
+	rhythmdb_property_model_delete (propmodel, entry);
+	g_hash_table_remove (propmodel->priv->entries, entry);
+	rhythmdb_property_model_sync (propmodel);
 }
 
 static gint
@@ -513,7 +543,7 @@ rhythmdb_property_model_compare (RhythmDBPropertyModelEntry *a, RhythmDBProperty
 	return strcmp (a_str, b_str);
 }
 
-static void
+static RhythmDBPropertyModelEntry *
 rhythmdb_property_model_insert (RhythmDBPropertyModel *model, RhythmDBEntry *entry)
 {
 	RhythmDBPropertyModelEntry *prop;
@@ -530,8 +560,9 @@ rhythmdb_property_model_insert (RhythmDBPropertyModel *model, RhythmDBEntry *ent
 		prop = g_sequence_ptr_get_data (ptr);
 		prop->refcount++;
 		rb_debug ("adding \"%s\": refcount %d", propstr, prop->refcount);
-		return;
+		return prop;
 	}
+	rb_debug ("adding new property \"%s\"", propstr);
 
 	prop = g_new0 (RhythmDBPropertyModelEntry, 1);
 	prop->string = rb_refstring_new (propstr);
@@ -550,15 +581,19 @@ rhythmdb_property_model_insert (RhythmDBPropertyModel *model, RhythmDBEntry *ent
 	path = rhythmdb_property_model_get_path (GTK_TREE_MODEL (model), &iter);
 	gtk_tree_model_row_inserted (GTK_TREE_MODEL (model), path, &iter);
 	gtk_tree_path_free (path);
+
+	return prop;
 }
 
-void
+static void
 rhythmdb_property_model_delete (RhythmDBPropertyModel *model,
 				RhythmDBEntry *entry)
 {
+	RhythmDBPropertyModelEntry *prop;
 	const char *propstr;
 
-	propstr = rhythmdb_entry_get_string (entry, model->priv->propid);
+	prop = g_hash_table_lookup (model->priv->entries, entry);
+	propstr = rb_refstring_get (prop->string);
 	rhythmdb_property_model_delete_prop (model, propstr);
 }
 	
@@ -602,13 +637,22 @@ rhythmdb_property_model_iter_from_string (RhythmDBPropertyModel *model,
 {
 	GSequencePtr ptr;
 
-	ptr = g_hash_table_lookup (model->priv->reverse_map, name);
+	if (name == NULL) {
+		if (iter) {
+			iter->stamp = model->priv->stamp;
+			iter->user_data = model->priv->all;
+		}
+		return TRUE;
+	}
 
+	ptr = g_hash_table_lookup (model->priv->reverse_map, name);
 	if (!ptr)
 		return FALSE;
 
-	iter->stamp = model->priv->stamp;
-	iter->user_data = ptr;
+	if (iter) {
+		iter->stamp = model->priv->stamp;
+		iter->user_data = ptr;
+	}
 
 	return TRUE;
 }
@@ -886,6 +930,10 @@ rhythmdb_property_model_drag_data_get (RbTreeDragSource *dragsource,
 		drag_target_list = rhythmdb_property_model_artist_drag_target_list;
 		sort_func = (GCompareDataFunc) rhythmdb_query_model_artist_sort_func;
 		break;
+	case RHYTHMDB_PROP_LOCATION:
+		drag_target_list = rhythmdb_property_model_location_drag_target_list;
+		sort_func = (GCompareDataFunc) rhythmdb_query_model_title_sort_func;
+		break;
 	default:
 		g_assert_not_reached ();
 	}
@@ -918,7 +966,8 @@ rhythmdb_property_model_drag_data_get (RbTreeDragSource *dragsource,
 		gtk_tree_path_free (path);
 		if (is_all) {
  			/*filter out radios*/
- 			rhythmdb_do_full_query (db, query_model,
+ 			rhythmdb_do_full_query (db, 
+						RHYTHMDB_QUERY_RESULTS (query_model),
  						RHYTHMDB_QUERY_PROP_EQUALS,
  						RHYTHMDB_PROP_TYPE, 
  						RHYTHMDB_ENTRY_TYPE_SONG,
@@ -951,7 +1000,8 @@ rhythmdb_property_model_drag_data_get (RbTreeDragSource *dragsource,
  				g_free (name);
  			}
  			/*filter out radios*/
- 			rhythmdb_do_full_query (db, query_model,
+ 			rhythmdb_do_full_query (db,
+						RHYTHMDB_QUERY_RESULTS (query_model),
  						RHYTHMDB_QUERY_PROP_EQUALS,
  						RHYTHMDB_PROP_TYPE, 
  						RHYTHMDB_ENTRY_TYPE_SONG,
@@ -1069,3 +1119,25 @@ rhythmdb_property_model_sync (RhythmDBPropertyModel *model)
 	model->priv->syncing_id = g_idle_add ((GSourceFunc)rhythmdb_property_model_perform_sync, model);
 }
 
+/* This should really be standard. */
+#define ENUM_ENTRY(NAME, DESC) { NAME, "" #NAME "", DESC }
+
+GType
+rhythmdb_property_model_column_get_type (void)
+{
+	static GType etype = 0;
+
+	if (etype == 0)	{
+		static const GEnumValue values[] = {
+			ENUM_ENTRY (RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE, "Property title"),
+			ENUM_ENTRY (RHYTHMDB_PROPERTY_MODEL_COLUMN_PRIORITY, "Value priority"),
+			ENUM_ENTRY (RHYTHMDB_PROPERTY_MODEL_COLUMN_NUMBER, "Track count"),
+			{ 0, 0, 0 }
+		};
+
+		etype = g_enum_register_static ("RhythmDBPropertyModelColumn", values);
+	}
+
+	return etype;
+}
+
