@@ -70,9 +70,12 @@
 #include <dbus/dbus-glib.h>
 #include "rb-shell-glue.h"
 #include "rb-shell-player-glue.h"
+#include "rb-playlist-manager.h"
+#include "rb-playlist-manager-glue.h"
 #endif
 
 static gboolean debug           = FALSE;
+static char *debug_match        = NULL;
 static gboolean quit            = FALSE;
 static gboolean no_registration = FALSE;
 static gboolean no_update	= FALSE;
@@ -134,6 +137,7 @@ main (int argc, char **argv)
 #if WITH_DBUS || WITH_BONOBO
 	GError *error = NULL;
 #endif
+	rb_profile_start ("starting rhythmbox");
 
 	struct poptOption popt_options[] =
 	{
@@ -167,8 +171,9 @@ main (int argc, char **argv)
 		{ "toggle-hide",		0,  POPT_ARG_NONE,	&toggle_hide,			0, N_("Change visibility of the main Rhythmbox window"), NULL },
 #endif
 
-		{ "debug",			'd',POPT_ARG_NONE,	&debug,				0, N_("Enable debugging code"), NULL },
-		{ "no-update",			0,  POPT_ARG_NONE,	&no_update,			0, N_("Do not update the library"), NULL },
+		{ "debug",			'd',POPT_ARG_NONE,	&debug,				0, N_("Enable debug output"), NULL },
+		{ "debug-match",		'D',POPT_ARG_STRING,	&debug_match,			0, N_("Enable debug output matching a specified string"), NULL },
+		{ "no-update",			0,  POPT_ARG_NONE,	&no_update,			0, N_("Do not update the library with file changes"), NULL },
 		{ "no-registration",		'n',POPT_ARG_NONE,	&no_registration,		0, N_("Do not register the shell"), NULL },
 		{ "dry-run",			0,  POPT_ARG_NONE,	&dry_run,			0, N_("Don't save any data permanently (implies --no-registration)"), NULL },
 		{ "rhythmdb-file",		0,  POPT_ARG_STRING,	&rhythmdb_file,			0, N_("Path for database file to use"), NULL },
@@ -193,25 +198,34 @@ main (int argc, char **argv)
         /* To pass options to GStreamer in 0.10, RB needs to use GOption, for
          * now, GStreamer can live without them (people can still use env
          * vars, after all) */
+	rb_profile_start ("initializing gstreamer");
         gst_init (NULL, NULL);
+	rb_profile_end ("initializing gstreamer");
 #endif
 #ifdef WITH_BONOBO
+	rb_profile_start ("initializing bonobo remote");
 	rb_remote_bonobo_preinit ();
+	rb_profile_end ("initializing bonobo remote");
 #endif
 
 	gtk_set_locale ();
+	rb_profile_start ("initializing gnome program");
 	program = gnome_program_init (PACKAGE, VERSION,
 				      LIBGNOMEUI_MODULE, argc+1, new_argv,
 				      GNOME_PARAM_POPT_TABLE, popt_options,
 				      GNOME_PARAM_HUMAN_READABLE_NAME, _("Rhythmbox"),
 				      GNOME_PARAM_APP_DATADIR, DATADIR,
 				      NULL);
+	rb_profile_end ("initializing gnome program");
+
 	g_object_get_property (G_OBJECT (program),
                                GNOME_PARAM_POPT_CONTEXT,
                                g_value_init (&context_as_value, G_TYPE_POINTER));
         poptContext = g_value_get_pointer (&context_as_value);
 
+	rb_profile_start ("initializing gnome auth manager");
 	gnome_authentication_manager_init ();
+	rb_profile_end ("initializing gnome auth manager");
 
 	g_random_set_seed (time(0));
 
@@ -226,7 +240,10 @@ main (int argc, char **argv)
 	gst_control_init (&argc, &argv);
 #endif
 
-	rb_debug_init (debug);
+	if (!debug && debug_match)
+		rb_debug_init_match (debug_match);
+	else
+		rb_debug_init (debug);
 	rb_debug ("initializing Rhythmbox %s", VERSION);
 	
 	/* TODO: kill this function */
@@ -235,6 +252,7 @@ main (int argc, char **argv)
 	activated = FALSE;
 
 #ifdef WITH_BONOBO
+	rb_profile_start ("creating bonobo remote");
 	rb_debug ("going to create Bonobo object");
 	bonobo = rb_remote_bonobo_new ();
 	client_proxy = NULL;
@@ -244,6 +262,7 @@ main (int argc, char **argv)
 			client_proxy = RB_REMOTE_CLIENT_PROXY (bonobo);
 		}
 	}
+	rb_profile_end ("creating bonobo remote");
 #endif
 #ifdef WITH_DBUS
 	rb_debug ("going to create DBus object");
@@ -315,7 +334,7 @@ main (int argc, char **argv)
 	
 		rb_stock_icons_init ();
 	
-		rb_shell = rb_shell_new (argc, argv, TRUE, no_update, dry_run, rhythmdb_file);
+		rb_shell = rb_shell_new (argc, argv, no_registration, no_update, dry_run, rhythmdb_file);
 		g_object_weak_ref (G_OBJECT (rb_shell), main_shell_weak_ref_cb, NULL);
 #ifdef WITH_BONOBO
 		if (!no_registration) {
@@ -333,20 +352,27 @@ main (int argc, char **argv)
 #endif
 #ifdef WITH_DBUS
 		if (!no_registration && session_bus != NULL) {
-			GObject *player;
+			GObject *obj;
 			const char *path;
 
 			dbus_g_object_type_install_info (RB_TYPE_SHELL, &dbus_glib_rb_shell_object_info);
-			
 			dbus_g_connection_register_g_object (session_bus, "/org/gnome/Rhythmbox/Shell", G_OBJECT (rb_shell));
+
+			/* register player object */
 			dbus_g_object_type_install_info (RB_TYPE_SHELL_PLAYER, &dbus_glib_rb_shell_player_object_info);
-			player = rb_shell_get_player (rb_shell);
+			obj = rb_shell_get_player (rb_shell);
 			path = rb_shell_get_player_path (rb_shell);
-			dbus_g_connection_register_g_object (session_bus, path, G_OBJECT (player));
+			dbus_g_connection_register_g_object (session_bus, path, obj);
+			
+			/* register playlist manager object */
+			dbus_g_object_type_install_info (RB_TYPE_PLAYLIST_MANAGER, &dbus_glib_rb_playlist_manager_object_info);
+			obj = rb_shell_get_playlist_manager (rb_shell);
+			path = rb_shell_get_playlist_manager_path (rb_shell);
+			dbus_g_connection_register_g_object (session_bus, path, obj);
 		}
 #endif
-	} else if (!no_registration) {
 #ifdef WITH_DBUS
+	} else if (!no_registration && session_bus != NULL) {
 		DBusGProxy *shell_proxy;
 		guint32 current_time;
 
@@ -393,12 +419,14 @@ main (int argc, char **argv)
 		gdk_notify_startup_complete ();
 	} else {
 
+		rb_profile_start ("mainloop");
 #ifdef WITH_BONOBO
                 /* Unfortunately Bonobo takes over the main loop ... */
 		bonobo_main ();
 #else
 		gtk_main ();
 #endif
+		rb_profile_end ("mainloop");
 
 		rb_debug ("out of toplevel loop");
 
@@ -412,6 +440,7 @@ main (int argc, char **argv)
 	g_strfreev (new_argv);
 
 	rb_debug ("THE END");
+	rb_profile_end ("starting rhythmbox");
 	exit (0);
 }
 
