@@ -1,8 +1,9 @@
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  *  arch-tag: Implementation of Rhythmbox Audioscrobbler support
  *
  *  Copyright (C) 2005 Alex Revo <xiphoidappendix@gmail.com>,
- *					   Ruben Vermeersch <ruben@Lambda1.be>
+ *		       Ruben Vermeersch <ruben@Lambda1.be>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,13 +22,15 @@
  */
 
 #include <errno.h>
-#include <gconf/gconf-value.h>
-#include <glib.h>
-#include <glib/gprintf.h>
-#include <gtk/gtk.h>
+
 #include <string.h>
 #include <time.h>
-#include <libgnome/gnome-i18n.h>
+
+#include <glib.h>
+#include <glib/gi18n.h>
+#include <glib/gprintf.h>
+#include <gtk/gtk.h>
+#include <gconf/gconf-value.h>
 
 #include <libsoup/soup.h>
 #include <libsoup/soup-uri.h>
@@ -43,7 +46,7 @@
 #include "rb-shell-player.h"
 #include "rb-source.h"
 #include "md5.h"
-
+#include "rb-proxy-config.h"
 
 
 #define CLIENT_ID "rbx"
@@ -53,7 +56,7 @@
 #define SCROBBLER_URL "http://post.audioscrobbler.com/"
 #define SCROBBLER_VERSION "1.1"
 
-#define EXTRA_URI_ENCODE_CHARS	"&"
+#define EXTRA_URI_ENCODE_CHARS	"&+"
 
 typedef struct
 {
@@ -120,22 +123,16 @@ struct _RBAudioscrobblerPrivate
 
 	/* HTTP requests session */
 	SoupSession *soup_session;
-
-	/* Proxy settings notifications (sigh, why do we need to monitor _SIX_ 
-	   fields?) */
-	guint notification_proxy_use_id;
-	guint notification_proxy_use_auth_id;
-	guint notification_proxy_host_id;
-	guint notification_proxy_port_id;
-	guint notification_proxy_user_id;
-	guint notification_proxy_password_id;
+	RBProxyConfig *proxy_config;
 };
 
+#define RB_AUDIOSCROBBLER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_AUDIOSCROBBLER, RBAudioscrobblerPrivate))
+
 
 static void	     audioscrobbler_entry_init (AudioscrobblerEntry *entry);
 static void	     audioscrobbler_entry_free (AudioscrobblerEntry *entry);
 
-static gboolean	 rb_audioscrobbler_load_queue (RBAudioscrobbler *audioscrobbler);
+static gboolean	     rb_audioscrobbler_load_queue (RBAudioscrobbler *audioscrobbler);
 static int	     rb_audioscrobbler_save_queue (RBAudioscrobbler *audioscrobbler);
 static void	     rb_audioscrobbler_print_queue (RBAudioscrobbler *audioscrobbler, gboolean submission);
 static void	     rb_audioscrobbler_free_queue_entries (RBAudioscrobbler *audioscrobbler, GSList **queue);
@@ -153,35 +150,37 @@ static void	     rb_audioscrobbler_set_property (GObject *object,
 static void	     rb_audioscrobbler_finalize (GObject *object);
 
 static void	     rb_audioscrobbler_add_timeout (RBAudioscrobbler *audioscrobbler);
-static gboolean	 rb_audioscrobbler_timeout_cb (RBAudioscrobbler *audioscrobbler);
+static gboolean	     rb_audioscrobbler_timeout_cb (RBAudioscrobbler *audioscrobbler);
 
-static gchar *	 mkmd5 (char *string);
+static gchar *	     mkmd5 (char *string);
 static int	     rb_audioscrobbler_parse_response (RBAudioscrobbler *audioscrobbler, SoupMessage *msg);
 static void	     rb_audioscrobbler_perform (RBAudioscrobbler *audioscrobbler,
 					       char *url,
 					       char *post_data,
 						   SoupMessageCallbackFn response_handler);
-static void		 rb_audioscrobbler_do_handshake (RBAudioscrobbler *audioscrobbler);
-static void		 rb_audioscrobbler_do_handshake_cb (SoupMessage *msg, gpointer user_data);
-static void		 rb_audioscrobbler_submit_queue (RBAudioscrobbler *audioscrobbler);
-static void		 rb_audioscrobbler_submit_queue_cb (SoupMessage *msg, gpointer user_data);
+static void	     rb_audioscrobbler_do_handshake (RBAudioscrobbler *audioscrobbler);
+static void	     rb_audioscrobbler_do_handshake_cb (SoupMessage *msg, gpointer user_data);
+static void	     rb_audioscrobbler_submit_queue (RBAudioscrobbler *audioscrobbler);
+static void	     rb_audioscrobbler_submit_queue_cb (SoupMessage *msg, gpointer user_data);
 
 static void	     rb_audioscrobbler_import_settings (RBAudioscrobbler *audioscrobbler);
 static void	     rb_audioscrobbler_preferences_sync (RBAudioscrobbler *audioscrobbler);
 
-
-static SoupUri * get_proxy_uri (void);
-static void		 rb_audioscrobbler_gconf_changed_cb (GConfClient *client,
-							    guint cnxn_id,
-					 		    GConfEntry *entry,
-					 		    RBAudioscrobbler *audioscrobbler);
-static void		 rb_audioscrobbler_song_changed_cb (RBShellPlayer *player,
-							    RhythmDBEntry *entry,
-							    RBAudioscrobbler *audioscrobbler);
+static void	     rb_audioscrobbler_gconf_changed_cb (GConfClient *client,
+							 guint cnxn_id,
+							 GConfEntry *entry,
+							 RBAudioscrobbler *audioscrobbler);
+static void	     rb_audioscrobbler_song_changed_cb (RBShellPlayer *player,
+							RhythmDBEntry *entry,
+							RBAudioscrobbler *audioscrobbler);
+static void	     rb_audioscrobbler_proxy_config_changed_cb (RBProxyConfig *config,
+								RBAudioscrobbler *audioscrobbler);
 enum
 {
 	PROP_0,
 	PROP_SHELL_PLAYER,
+	PROP_SHELL_PREFS,
+	PROP_PROXY_CONFIG
 };
 
 enum
@@ -213,23 +212,37 @@ rb_audioscrobbler_class_init (RBAudioscrobblerClass *klass)
 
 	g_object_class_install_property (object_class,
 					 PROP_SHELL_PLAYER,
-					 g_param_spec_object ("shell_player",
+					 g_param_spec_object ("shell-player",
 							      "RBShellPlayer",
 							      "RBShellPlayer object",
 							      RB_TYPE_SHELL_PLAYER,
-							      G_PARAM_READWRITE));
+							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (object_class,
+					 PROP_SHELL_PREFS,
+					 g_param_spec_object ("shell-preferences",
+							      "RBShellPreferences",
+							      "RBShellPreferences object",
+							      RB_TYPE_SHELL_PREFERENCES,
+							      G_PARAM_WRITABLE));
+	g_object_class_install_property (object_class,
+					 PROP_PROXY_CONFIG,
+					 g_param_spec_object ("proxy-config",
+							      "RBProxyConfig",
+							      "RBProxyConfig object",
+							      RB_TYPE_PROXY_CONFIG,
+							      G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+
+	g_type_class_add_private (klass, sizeof (RBAudioscrobblerPrivate));
 }
 
 static void
 rb_audioscrobbler_init (RBAudioscrobbler *audioscrobbler)
 {
-	SoupUri *uri = NULL;
-
 	rb_debug ("Initialising Audioscrobbler");
 	rb_debug ("Plugin ID: %s, Version %s (Protocol %s)",
 		  CLIENT_ID, CLIENT_VERSION, SCROBBLER_VERSION);
 
-	audioscrobbler->priv = g_new0 (RBAudioscrobblerPrivate, 1);
+	audioscrobbler->priv = RB_AUDIOSCROBBLER_GET_PRIVATE (audioscrobbler);
 
 	audioscrobbler->priv->queue = NULL;
 	audioscrobbler->priv->submission= NULL;
@@ -250,21 +263,9 @@ rb_audioscrobbler_init (RBAudioscrobbler *audioscrobbler)
 	audioscrobbler->priv->elapsed = 0;
 	audioscrobbler->priv->timeout_id = 0;
 
-	/* Setup soup session, with proxy if needed */
-	uri = get_proxy_uri ();
-	if (uri) {
-		audioscrobbler->priv->soup_session = soup_session_async_new_with_options (
-					"proxy-uri", uri,
-					NULL);
-		soup_uri_free (uri);
-	} else {
-		audioscrobbler->priv->soup_session = soup_session_async_new ();
-	}
-
 	rb_audioscrobbler_load_queue (audioscrobbler);
 
 	rb_audioscrobbler_import_settings (audioscrobbler);
-	rb_audioscrobbler_get_config_widget (audioscrobbler);
 
 	/* gconf notifications: */
 	audioscrobbler->priv->notification_username_id = 
@@ -279,32 +280,6 @@ rb_audioscrobbler_init (RBAudioscrobbler *audioscrobbler)
 		eel_gconf_notification_add (CONF_AUDIOSCROBBLER_ENABLED,
 				    (GConfClientNotifyFunc) rb_audioscrobbler_gconf_changed_cb,
 				    audioscrobbler);
-
-	/* proxy notifications: */
-	audioscrobbler->priv->notification_proxy_use_id =
-		eel_gconf_notification_add ("/system/http_proxy/use_http_proxy",
-					(GConfClientNotifyFunc) rb_audioscrobbler_gconf_changed_cb,
-					audioscrobbler);
-	audioscrobbler->priv->notification_proxy_use_auth_id =
-		eel_gconf_notification_add ("/system/http_proxy/use_authentication",
-					(GConfClientNotifyFunc) rb_audioscrobbler_gconf_changed_cb,
-					audioscrobbler);
-	audioscrobbler->priv->notification_proxy_host_id =
-		eel_gconf_notification_add ("/system/http_proxy/host",
-					(GConfClientNotifyFunc) rb_audioscrobbler_gconf_changed_cb,
-					audioscrobbler);
-	audioscrobbler->priv->notification_proxy_port_id =
-		eel_gconf_notification_add ("/system/http_proxy/port",
-					(GConfClientNotifyFunc) rb_audioscrobbler_gconf_changed_cb,
-					audioscrobbler);
-	audioscrobbler->priv->notification_proxy_user_id =
-		eel_gconf_notification_add ("/system/http_proxy/authentication_user",
-					(GConfClientNotifyFunc) rb_audioscrobbler_gconf_changed_cb,
-					audioscrobbler);
-	audioscrobbler->priv->notification_proxy_password_id =
-		eel_gconf_notification_add ("/system/http_proxy/authentication_password",
-					(GConfClientNotifyFunc) rb_audioscrobbler_gconf_changed_cb,
-					audioscrobbler);
 }
 
 static void
@@ -328,13 +303,6 @@ rb_audioscrobbler_finalize (GObject *object)
 	eel_gconf_notification_remove (audioscrobbler->priv->notification_password_id);
 	eel_gconf_notification_remove (audioscrobbler->priv->notification_enabled_id);
 
-	eel_gconf_notification_remove (audioscrobbler->priv->notification_proxy_use_id);
-	eel_gconf_notification_remove (audioscrobbler->priv->notification_proxy_use_auth_id);
-	eel_gconf_notification_remove (audioscrobbler->priv->notification_proxy_host_id);
-	eel_gconf_notification_remove (audioscrobbler->priv->notification_proxy_port_id);
-	eel_gconf_notification_remove (audioscrobbler->priv->notification_proxy_user_id);
-	eel_gconf_notification_remove (audioscrobbler->priv->notification_proxy_password_id);
-
 	g_free (audioscrobbler->priv->md5_challenge);
 	g_free (audioscrobbler->priv->username);
 	g_free (audioscrobbler->priv->password);
@@ -342,20 +310,24 @@ rb_audioscrobbler_finalize (GObject *object)
 	g_free (audioscrobbler->priv->artist);
 	g_free (audioscrobbler->priv->album);
 	g_free (audioscrobbler->priv->title);
-	g_free (audioscrobbler->priv->soup_session);
+	if (audioscrobbler->priv->soup_session)
+		g_object_unref (G_OBJECT (audioscrobbler->priv->soup_session));
+	g_object_unref (G_OBJECT (audioscrobbler->priv->proxy_config));
+	g_object_unref (G_OBJECT (audioscrobbler->priv->shell_player));
 
 	rb_audioscrobbler_free_queue_entries (audioscrobbler, &audioscrobbler->priv->queue);
 	rb_audioscrobbler_free_queue_entries (audioscrobbler, &audioscrobbler->priv->submission);
-	g_free (audioscrobbler->priv);
 
 	G_OBJECT_CLASS (rb_audioscrobbler_parent_class)->finalize (object);
 }
 
-RBAudioscrobbler *
-rb_audioscrobbler_new (RBShellPlayer *shell_player)
+RBAudioscrobbler*
+rb_audioscrobbler_new (RBShellPlayer *shell_player,
+		       RBProxyConfig *proxy_config)
 {
 	return g_object_new (RB_TYPE_AUDIOSCROBBLER,
-			     "shell_player", shell_player,
+			     "shell-player", shell_player,
+			     "proxy-config", proxy_config,
 			     NULL);
 }
 
@@ -370,14 +342,33 @@ rb_audioscrobbler_set_property (GObject *object,
 	switch (prop_id) {
 		case PROP_SHELL_PLAYER:
 			audioscrobbler->priv->shell_player = g_value_get_object (value);
+			g_object_ref (G_OBJECT (audioscrobbler->priv->shell_player));
 			g_signal_connect_object (G_OBJECT (audioscrobbler->priv->shell_player),
 						 "playing-song-changed",
 						 G_CALLBACK (rb_audioscrobbler_song_changed_cb),
 						 audioscrobbler, 0);
 			break;
+		case PROP_SHELL_PREFS:
+		{
+			RBShellPreferences *prefs = g_value_get_object (value);
+			GtkWidget *config_widget = rb_audioscrobbler_get_config_widget (audioscrobbler);
 
+			rb_shell_preferences_append_page (prefs,
+				       			  _("Audioscrobbler"),
+							  config_widget);
+			break;
+		}
+		case PROP_PROXY_CONFIG:
+			audioscrobbler->priv->proxy_config = g_value_get_object (value);
+			g_object_ref (G_OBJECT (audioscrobbler->priv->proxy_config));
+			g_signal_connect_object (G_OBJECT (audioscrobbler->priv->proxy_config),
+						 "config-changed",
+						 G_CALLBACK (rb_audioscrobbler_proxy_config_changed_cb),
+						 audioscrobbler, 0);
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+			break;
 	}
 }
 
@@ -393,9 +384,9 @@ rb_audioscrobbler_get_property (GObject *object,
 		case PROP_SHELL_PLAYER:
 			g_value_set_object (value, audioscrobbler->priv->shell_player);
 			break;
-
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+			break;
 	}
 }
 
@@ -525,10 +516,14 @@ rb_audioscrobbler_parse_response (RBAudioscrobbler *audioscrobbler, SoupMessage
 	rb_debug ("Parsing response, status=%d", msg->status_code);
 
 	if (SOUP_STATUS_IS_SUCCESSFUL (msg->status_code) && (msg->response).body != NULL) {
+		gchar *body;
 		gchar **breaks;
 		
-		g_strstrip ((msg->response).body);
-		breaks = g_strsplit ((msg->response).body, "\n", 4);
+		body = g_malloc0 ((msg->response).length + 1);
+		memcpy (body, (msg->response).body, (msg->response).length);
+		
+		g_strstrip (body);
+		breaks = g_strsplit (body, "\n", 4);
 		int i;
 
 		for (i = 0; breaks[i] != NULL; i++) {
@@ -601,6 +596,7 @@ rb_audioscrobbler_parse_response (RBAudioscrobbler *audioscrobbler, SoupMessage
 			audioscrobbler->priv->submit_next = time(NULL) + audioscrobbler->priv->submit_interval;
 
 		g_strfreev (breaks);
+		g_free (body);
 	}
 
 	if (ret_val != RESP_0)
@@ -629,6 +625,18 @@ rb_audioscrobbler_perform (RBAudioscrobbler *audioscrobbler,
 					  strlen (post_data));
 	}
 
+	/* create soup session, if we haven't got one yet */
+	if (!audioscrobbler->priv->soup_session) {
+		SoupUri *uri;
+		
+		uri = rb_proxy_config_get_libsoup_uri (audioscrobbler->priv->proxy_config);
+		audioscrobbler->priv->soup_session = soup_session_async_new_with_options (
+					"proxy-uri", uri,
+					NULL);
+		if (uri)
+			soup_uri_free (uri);
+	}
+	
 	soup_session_queue_message (audioscrobbler->priv->soup_session,
 				    msg,
 				    response_handler,
@@ -819,8 +827,8 @@ rb_audioscrobbler_import_settings (RBAudioscrobbler *audioscrobbler)
 	/* import gconf settings. */
 	g_free (audioscrobbler->priv->username);
 	g_free (audioscrobbler->priv->password);
-	audioscrobbler->priv->username = g_strdup (eel_gconf_get_string (CONF_AUDIOSCROBBLER_USERNAME));
-	audioscrobbler->priv->password = g_strdup (eel_gconf_get_string (CONF_AUDIOSCROBBLER_PASSWORD));
+	audioscrobbler->priv->username = eel_gconf_get_string (CONF_AUDIOSCROBBLER_USERNAME);
+	audioscrobbler->priv->password = eel_gconf_get_string (CONF_AUDIOSCROBBLER_PASSWORD);
 	audioscrobbler->priv->enabled =  eel_gconf_get_boolean (CONF_AUDIOSCROBBLER_ENABLED);
 
 	if (eel_gconf_get_boolean (CONF_AUDIOSCROBBLER_ENABLED))
@@ -844,9 +852,6 @@ GtkWidget *
 rb_audioscrobbler_get_config_widget (RBAudioscrobbler *audioscrobbler)
 {
 	GladeXML *xml;
-	GtkWidget *audioscrobbler_label;
-	PangoAttrList *pattrlist;
-	PangoAttribute *attr;
 
 	if (audioscrobbler->priv->config_widget) {
 		return audioscrobbler->priv->config_widget;
@@ -860,14 +865,7 @@ rb_audioscrobbler_get_config_widget (RBAudioscrobbler *audioscrobbler)
 	audioscrobbler->priv->password_label = glade_xml_get_widget (xml, "password_label");
 	audioscrobbler->priv->enabled_check = glade_xml_get_widget (xml, "enabled_check");
 
-	pattrlist = pango_attr_list_new (); 
-	attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
-	attr->start_index = 0;
-	attr->end_index = G_MAXINT; 
-	pango_attr_list_insert (pattrlist, attr);
-	audioscrobbler_label = glade_xml_get_widget (xml, "audioscrobbler_label");
-	gtk_label_set_attributes (GTK_LABEL (audioscrobbler_label), pattrlist);
-	pango_attr_list_unref (pattrlist); 
+	rb_glade_boldify_label (xml, "audioscrobbler_label");
 
 	g_object_unref (G_OBJECT (xml));
 
@@ -878,27 +876,21 @@ rb_audioscrobbler_get_config_widget (RBAudioscrobbler *audioscrobbler)
 
 
 /* Callback functions: */
-static SoupUri *
-get_proxy_uri ()
+
+static void
+rb_audioscrobbler_proxy_config_changed_cb (RBProxyConfig *config,
+					   RBAudioscrobbler *audioscrobbler)
 {
-	SoupUri *uri = NULL;
+	SoupUri *uri;
 
-	if (!eel_gconf_get_boolean ("/system/http_proxy/use_http_proxy")) {
-		return NULL;
+	if (audioscrobbler->priv->soup_session) {
+		uri = rb_proxy_config_get_libsoup_uri (config);
+		g_object_set (G_OBJECT (audioscrobbler->priv->soup_session),
+					"proxy-uri", uri,
+					NULL);
+		if (uri)
+			soup_uri_free (uri);
 	}
-	
-	uri = g_new0 (SoupUri, 1);
-	uri->protocol = SOUP_PROTOCOL_HTTP;
-
-	uri->host = g_strdup (eel_gconf_get_string ("/system/http_proxy/host"));
-	uri->port = eel_gconf_get_integer ("/system/http_proxy/port");
-
-	if (eel_gconf_get_boolean ("/system/http_proxy/use_authentication")) {
-		uri->user = g_strdup (eel_gconf_get_string ("/system/http_proxy/authentication_user"));
-		uri->passwd = g_strdup (eel_gconf_get_string ("/system/http_proxy/authentication_password"));
-	}	
-
-	return uri;
 }
 
 static void
@@ -911,15 +903,19 @@ rb_audioscrobbler_gconf_changed_cb (GConfClient *client,
 	if (strcmp (entry->key, CONF_AUDIOSCROBBLER_USERNAME) == 0) {
 		g_free (audioscrobbler->priv->username);
 		audioscrobbler->priv->username = g_strdup (gconf_value_get_string (entry->value));
-		gtk_entry_set_text (GTK_ENTRY (audioscrobbler->priv->username_entry),
-				    gconf_value_get_string (entry->value));
+
+		if (audioscrobbler->priv->username_entry)
+			gtk_entry_set_text (GTK_ENTRY (audioscrobbler->priv->username_entry),
+					    gconf_value_get_string (entry->value));
 
 		audioscrobbler->priv->handshake = FALSE;
 	} else if (strcmp (entry->key, CONF_AUDIOSCROBBLER_PASSWORD) == 0) {
 		g_free (audioscrobbler->priv->password);
 		audioscrobbler->priv->password = g_strdup (gconf_value_get_string (entry->value));
-		gtk_entry_set_text (GTK_ENTRY (audioscrobbler->priv->password_entry),
-				    gconf_value_get_string (entry->value));
+	
+		if (audioscrobbler->priv->password_entry)
+			gtk_entry_set_text (GTK_ENTRY (audioscrobbler->priv->password_entry),
+					    gconf_value_get_string (entry->value));
 	} else if (strcmp (entry->key, CONF_AUDIOSCROBBLER_ENABLED) == 0) {
 		audioscrobbler->priv->enabled = gconf_value_get_bool (entry->value);
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (audioscrobbler->priv->enabled_check),
@@ -927,13 +923,6 @@ rb_audioscrobbler_gconf_changed_cb (GConfClient *client,
 
 		if (gconf_value_get_bool (entry->value))
 			rb_audioscrobbler_add_timeout (audioscrobbler);
-	} else if (g_str_has_prefix (entry->key, "/system/http_proxy/")) {
-		/* Update proxy */
-		SoupUri *uri = get_proxy_uri ();
-		g_object_set (G_OBJECT (audioscrobbler->priv->soup_session),
-					 "proxy-uri", uri,
-					 NULL);
-		soup_uri_free (uri);
 	} else {
 		rb_debug ("Unhandled GConf key updated: \"%s\"", entry->key);
 	}
@@ -944,32 +933,37 @@ rb_audioscrobbler_song_changed_cb (RBShellPlayer *player,
 				   RhythmDBEntry *entry,
 				   RBAudioscrobbler *audioscrobbler)
 {
+	gulong type;
 	gchar *old_artist;
 	gchar *old_album;
 	gchar *old_title;
 
 	if (entry == NULL)
 		return;
-	if (entry->type == RHYTHMDB_ENTRY_TYPE_IRADIO_STATION ||
-	    entry->type == RHYTHMDB_ENTRY_TYPE_PODCAST_POST ||
-	    entry->type == RHYTHMDB_ENTRY_TYPE_PODCAST_FEED)
+
+	type = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TYPE);
+	if (type == RHYTHMDB_ENTRY_TYPE_IRADIO_STATION ||
+	    type == RHYTHMDB_ENTRY_TYPE_PODCAST_POST ||
+	    type == RHYTHMDB_ENTRY_TYPE_PODCAST_FEED)
+		return;
+
+	if (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_PLAYBACK_ERROR) != NULL)
 		return;
 
 	old_artist = audioscrobbler->priv->artist;
 	old_album = audioscrobbler->priv->album;
 	old_title = audioscrobbler->priv->title;
 
-	audioscrobbler->priv->artist = g_strdup (rb_refstring_get (entry->artist));
-	audioscrobbler->priv->album = g_strdup (rb_refstring_get (entry->album));
-	audioscrobbler->priv->title = g_strdup (rb_refstring_get (entry->title));
-	audioscrobbler->priv->duration = (int) entry->duration;
+	audioscrobbler->priv->title = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_TITLE);
+	audioscrobbler->priv->artist = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_ARTIST);
+	audioscrobbler->priv->album = rhythmdb_entry_dup_string (entry, RHYTHMDB_PROP_ALBUM);
+	audioscrobbler->priv->duration = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DURATION);
 	guint time;
 	rb_shell_player_get_playing_time (audioscrobbler->priv->shell_player, &time, NULL);
 	audioscrobbler->priv->elapsed = (int) time;
 
 	/* Ignore tracks that violate the specification (v1.1) */
 	if (audioscrobbler->priv->duration < 30 ||
-	    audioscrobbler->priv->duration > 1800 ||
 	    ! strcmp (audioscrobbler->priv->artist, _("Unknown")) ||
 	    ! strcmp (audioscrobbler->priv->title, _("Unknown"))) {
 		audioscrobbler->priv->should_queue = FALSE;
@@ -1021,26 +1015,6 @@ rb_audioscrobbler_enabled_check_changed_cb (GtkCheckButton *button,
 {
 	eel_gconf_set_boolean (CONF_AUDIOSCROBBLER_ENABLED,
 			       gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)));
-
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button))) {
-		gtk_widget_set_sensitive (GTK_WIDGET (audioscrobbler->priv->username_entry),
-					  TRUE);
-		gtk_widget_set_sensitive (GTK_WIDGET (audioscrobbler->priv->username_label),
-					  TRUE);
-		gtk_widget_set_sensitive (GTK_WIDGET (audioscrobbler->priv->password_entry),
-					  TRUE);
-		gtk_widget_set_sensitive (GTK_WIDGET (audioscrobbler->priv->password_label),
-					  TRUE);
-	} else {
-		gtk_widget_set_sensitive (GTK_WIDGET (audioscrobbler->priv->username_entry),
-					  FALSE);
-		gtk_widget_set_sensitive (GTK_WIDGET (audioscrobbler->priv->username_label),
-					  FALSE);
-		gtk_widget_set_sensitive (GTK_WIDGET (audioscrobbler->priv->password_entry),
-					  FALSE);
-		gtk_widget_set_sensitive (GTK_WIDGET (audioscrobbler->priv->password_label),
-					  FALSE);
-	}
 }
 
 
@@ -1128,16 +1102,18 @@ rb_audioscrobbler_load_entry_from_string (const char *string)
 static gboolean
 rb_audioscrobbler_load_queue (RBAudioscrobbler *audioscrobbler)
 {
-	char *pathname;
+	char *pathname, *uri;
 	GnomeVFSResult result;
 	char *data;
 	int size;
 
 	pathname = g_build_filename (rb_dot_dir (), "audioscrobbler.queue", NULL);
-	rb_debug ("Loading Audioscrobbler queue from \"%s\"", pathname);
-
-	result = gnome_vfs_read_entire_file (pathname, &size, &data);
+	uri = g_filename_to_uri (pathname, NULL, NULL);
 	g_free (pathname);
+	rb_debug ("Loading Audioscrobbler queue from \"%s\"", uri);
+
+	result = gnome_vfs_read_entire_file (uri, &size, &data);
+	g_free (uri);
 	
 	/* do stuff */
 	if (result == GNOME_VFS_OK) {
