@@ -47,6 +47,9 @@ typedef enum
         RB_SHELL_PLAYER_ERROR_NOT_SEEKABLE,
 } RBShellPlayerError;
 
+GType rb_shell_player_error_get_type (void);
+#define RB_TYPE_SHELL_PLAYER_ERROR	(rb_shell_player_error_get_type())
+
 #define RB_SHELL_PLAYER_ERROR rb_shell_player_error_quark ()
 
 GQuark rb_shell_player_error_quark (void);
@@ -70,7 +73,6 @@ typedef struct
 	void (*playing_source_changed) (RBShellPlayer *player, RBSource *source);
 	void (*playing_uri_changed) (RBShellPlayer *player, const char *uri);
 	void (*playing_song_changed) (RBShellPlayer *player, RhythmDBEntry *entry);
-	void (*playing_from_queue) (RBShellPlayer *player, gboolean from_queue);
 } RBShellPlayerClass;
 
 GType			rb_shell_player_get_type   (void);
@@ -85,11 +87,14 @@ void			rb_shell_player_set_playing_source (RBShellPlayer *player,
 							    RBSource *source);
 
 RBSource *		rb_shell_player_get_playing_source (RBShellPlayer *shell_player);
+RBSource *		rb_shell_player_get_active_source (RBShellPlayer *shell_player);
 
 void			rb_shell_player_jump_to_current (RBShellPlayer *player);
 
 void			rb_shell_player_play_entry	(RBShellPlayer *player,
 							 RhythmDBEntry *entry);
+gboolean		rb_shell_player_play		(RBShellPlayer *player, GError **error);
+gboolean		rb_shell_player_pause		(RBShellPlayer *player, GError **error);
 gboolean                rb_shell_player_playpause	(RBShellPlayer *player, gboolean ignore_stop, GError **error);
 void			rb_shell_player_stop		(RBShellPlayer *player);
 gboolean                rb_shell_player_do_previous	(RBShellPlayer *player, GError **error);
@@ -113,8 +118,6 @@ gboolean		rb_shell_player_get_playing_path(RBShellPlayer *shell_player,
 							 const gchar **path,
 							 GError **error);
 
-void			rb_shell_player_sync_buttons	(RBShellPlayer *player);
-
 void			rb_shell_player_set_playback_state(RBShellPlayer *player,
 							   gboolean shuffle, gboolean repeat);
 
@@ -147,10 +150,6 @@ gboolean		rb_shell_player_get_mute	(RBShellPlayer *player,
 							 GError **error);
 
 
-#ifdef HAVE_ACME
-gboolean		rb_shell_player_handle_key	(RBShellPlayer *player, guint keyval);
-#endif
-
 G_END_DECLS
 
 #endif /* __RB_SHELL_PLAYER_H */
