@@ -23,7 +23,6 @@
 /*
  * TODO
  *    * handle cases where MusicBrainz returns multiple albums
- *    * use release-date metadata (can have multiple values)
  */
 
 #include <config.h>
@@ -57,8 +56,9 @@ static GObject *rb_audiocd_source_constructor (GType type, guint n_construct_pro
 
 static gboolean impl_show_popup (RBSource *source);
 static void impl_delete_thyself (RBSource *source);
+static GList* impl_get_ui_actions (RBSource *source);
 
-static gboolean rb_audiocd_load_songs (RBAudioCdSource *source);
+static gpointer rb_audiocd_load_songs (RBAudioCdSource *source);
 static void rb_audiocd_load_metadata (RBAudioCdSource *source, RhythmDB *db);
 static void rb_audiocd_load_metadata_cancel (RBAudioCdSource *source);
 
@@ -91,9 +91,13 @@ rb_audiocd_source_class_init (RBAudioCdSourceClass *klass)
 	/* don't bother showing the browser/search bits */
 	source_class->impl_can_search = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_browse = (RBSourceFeatureFunc) rb_false_function;
+	source_class->impl_can_paste = (RBSourceFeatureFunc) rb_false_function;
+	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_false_function;
+	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_true_function;
 
 	source_class->impl_show_popup = impl_show_popup;
 	source_class->impl_delete_thyself = impl_delete_thyself;
+	source_class->impl_get_ui_actions = impl_get_ui_actions;
 
 	g_type_class_add_private (klass, sizeof (RBAudioCdSourcePrivate));
 }
@@ -131,14 +135,20 @@ static GObject *
 rb_audiocd_source_constructor (GType type, guint n_construct_properties,
 			       GObjectConstructParam *construct_properties)
 {
-	GObjectClass *klass, *parent_class; 
 	RBAudioCdSource *source; 
+	RBEntryView *entry_view;
 
-	klass = G_OBJECT_CLASS (g_type_class_peek (type));
-	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
-	source = RB_AUDIOCD_SOURCE (parent_class->constructor (type, n_construct_properties, construct_properties));
+	source = RB_AUDIOCD_SOURCE (G_OBJECT_CLASS (rb_audiocd_source_parent_class)->
+			constructor (type, n_construct_properties, construct_properties));
 
-	g_idle_add ((GSourceFunc)rb_audiocd_load_songs, source);
+	g_object_set (G_OBJECT (source), "name", "Unknown Audio", NULL);
+
+	/* we want audio cds to sort by track# by default */
+	entry_view = rb_source_get_entry_view (RB_SOURCE (source));
+	rb_entry_view_set_sorting_order (entry_view, "Track", GTK_SORT_ASCENDING);
+
+	g_object_ref (G_OBJECT (source));
+	g_thread_create ((GThreadFunc)rb_audiocd_load_songs, source, FALSE, NULL);
 
 	return G_OBJECT (source);
 }
@@ -179,6 +189,11 @@ entry_set_string_prop (RhythmDB *db, RhythmDBEntry *entry, gboolean is_inserted,
 	if (!str)
 		str = _("Unknown");
 
+	if (!g_utf8_validate (str, -1, NULL)) {
+		rb_debug ("Got invalid UTF-8 tag data");
+		str = _("<Invalid unicode>");
+	}
+
 	g_value_init (&value, G_TYPE_STRING);
 	g_value_set_string (&value, str);
 	if (is_inserted)
@@ -198,7 +213,7 @@ rb_audiocd_create_track_entry (RBAudioCdSource *source, RhythmDB *db, guint trac
 	gchar *str;
 	RhythmDBEntryType entry_type;
 
-	audio_path = g_strdup_printf ("cdda://%s:%d", priv->device_path, track_number);
+	audio_path = g_strdup_printf ("cdda://%d#%s", track_number, priv->device_path);
 
 	g_object_get (G_OBJECT (source), "entry-type", &entry_type, NULL);
 	rb_debug ("Audio CD - create entry for track %d from %s", track_number, audio_path);
@@ -262,6 +277,7 @@ rb_audiocd_create_track_entry (RBAudioCdSource *source, RhythmDB *db, guint trac
 	entry_set_string_prop (db, entry, FALSE, RHYTHMDB_PROP_ARTIST, NULL);
 	entry_set_string_prop (db, entry, FALSE, RHYTHMDB_PROP_ALBUM, NULL);
 	entry_set_string_prop (db, entry, FALSE, RHYTHMDB_PROP_GENRE, NULL);
+	entry_set_string_prop (db, entry, FALSE, RHYTHMDB_PROP_MIMETYPE, "audio/x-raw-int");
 
 	rhythmdb_commit (db);
 	g_free (audio_path);
@@ -401,7 +417,17 @@ metadata_cb (SjMetadata *metadata, GList *albums, GError *error, RBAudioCdSource
 			rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_DURATION, &value);
 			g_value_unset (&value);
 			
-			/*album->release_date (has multiple values)*/
+			/*album->release_date (could potentially have multiple values)*/
+			/* in current sj-structures.h, however, it does not */
+			
+			if (album->release_date) {
+				GType type = rhythmdb_get_property_type (db, RHYTHMDB_PROP_DATE);
+				g_value_init (&value, type);
+				g_value_set_ulong (&value, g_date_get_julian (album->release_date));
+				rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_DATE, &value);
+				g_value_unset (&value);
+			}
+
 			rhythmdb_commit (db);
 
 			album->tracks = g_list_next (album->tracks);
@@ -465,14 +491,13 @@ rb_audiocd_load_metadata_cancel (RBAudioCdSource *source)
 #endif
 }
 
-static gboolean
+static gpointer
 rb_audiocd_load_songs (RBAudioCdSource *source)
 {
 	RBAudioCdSourcePrivate *priv = AUDIOCD_SOURCE_GET_PRIVATE (source);
 	RBShell *shell;
 	RhythmDB *db;
 	GnomeVFSVolume *volume;
-	RBEntryView *entry_view;
 	
 	g_object_get (G_OBJECT (source), "volume", &volume, NULL);
 	priv->device_path = gnome_vfs_volume_get_device_path (volume);
@@ -482,12 +507,6 @@ rb_audiocd_load_songs (RBAudioCdSource *source)
 	g_object_get (G_OBJECT (shell), "db", &db, NULL);
 	g_object_unref (G_OBJECT (shell));
 
-
-	/* we want audio cds to sort by track# by default */
-	entry_view = rb_source_get_entry_view (RB_SOURCE (source));
-	rb_entry_view_set_sorting_order (entry_view, "Track", GTK_SORT_ASCENDING);
-
-
 	rb_debug ("loading Audio CD from %s", priv->device_path);
 	/* create a cdda gstreamer element, to get cd info from */
 	priv->cdda = gst_element_make_from_uri (GST_URI_SRC, "cdda://", NULL);
@@ -514,8 +533,9 @@ rb_audiocd_load_songs (RBAudioCdSource *source)
 
 error_out:
 	g_object_unref (G_OBJECT (db));
+	g_object_unref (G_OBJECT (source));
 
-	return FALSE;
+	return NULL;
 }
 
 static void
@@ -579,3 +599,16 @@ impl_show_popup (RBSource *source)
 	return TRUE;
 }
 
+static GList*
+impl_get_ui_actions (RBSource *source)
+{
+	GList *actions = NULL;
+
+#ifdef ENABLE_TRACK_TRANSFER
+	actions = g_list_prepend (actions, g_strdup ("RemovableSourceCopyAllTracks"));
+#endif
+	actions = g_list_prepend (actions, g_strdup ("RemovableSourceEject"));
+
+	return actions;
+}
+
