@@ -38,7 +38,8 @@
 #include "rb-preferences.h"
 
 #include "rb-daap-connection.h"
-#include "rb-daap-mdns.h"
+#include "rb-daap-mdns-browser.h"
+#include "rb-daap-dialog.h"
 
 #include "rb-static-playlist-source.h"
 
@@ -58,8 +59,8 @@ static void rb_daap_source_connection_cb (RBDAAPConnection *connection,
 					  RBSource *source);
 static gboolean rb_daap_source_disconnect (RBSource *source);
 static gboolean rb_daap_source_show_popup (RBSource *source);
-static const gchar * rb_daap_source_get_browser_key (RBSource *source);
-static const gchar * rb_daap_source_get_paned_key (RBLibrarySource *source);
+static char * rb_daap_source_get_browser_key (RBSource *source);
+static const char * rb_daap_source_get_paned_key (RBBrowserSource *source);
 
 
 #define CONF_ENABLE_BROWSING CONF_PREFIX "/sharing/enable_browsing"
@@ -68,16 +69,19 @@ static const gchar * rb_daap_source_get_paned_key (RBLibrarySource *source);
 #define CONF_STATE_SHOW_BROWSER CONF_PREFIX "/state/daap/show_browser"
 
 
-static RBDAAPmDNSBrowser browser = 0;
-static GHashTable *service_name_to_resolver = NULL;
-static GSList *sources = NULL;
+static RBDaapMdnsBrowser *mdns_browser = NULL;
+
+static GHashTable *source_lookup = NULL;
+
 static guint enable_browsing_notify_id = EEL_GCONF_UNDEFINED_CONNECTION;
 
+static GdkPixbuf *daap_share_pixbuf        = NULL;
+static GdkPixbuf *daap_share_pixbuf_locked = NULL;
 
 struct RBDAAPSourcePrivate
 {
-	gchar *service_name;
-	gchar *host;
+	char *service_name;
+	char *host;
 	guint port;
 	gboolean password_protected;
 
@@ -95,7 +99,7 @@ enum {
 	PROP_PASSWORD_PROTECTED
 };
 
-G_DEFINE_TYPE (RBDAAPSource, rb_daap_source, RB_TYPE_LIBRARY_SOURCE)
+G_DEFINE_TYPE (RBDAAPSource, rb_daap_source, RB_TYPE_BROWSER_SOURCE)
 
 
 static RhythmDBEntryType
@@ -109,7 +113,7 @@ rb_daap_source_class_init (RBDAAPSourceClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
-	RBLibrarySourceClass *library_source_class = RB_LIBRARY_SOURCE_CLASS (klass);
+	RBBrowserSourceClass *browser_source_class = RB_BROWSER_SOURCE_CLASS (klass);
 
 	object_class->dispose = rb_daap_source_dispose;
 	object_class->get_property = rb_daap_source_get_property;
@@ -119,7 +123,7 @@ rb_daap_source_class_init (RBDAAPSourceClass *klass)
 	source_class->impl_disconnect = rb_daap_source_disconnect;
 	source_class->impl_can_search = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_false_function;
-	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_false_function;
+	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_paste = NULL;
 	source_class->impl_receive_drag = NULL;
@@ -128,9 +132,9 @@ rb_daap_source_class_init (RBDAAPSourceClass *klass)
 	source_class->impl_get_config_widget = NULL;
 	source_class->impl_get_browser_key = rb_daap_source_get_browser_key;
 
-	library_source_class->impl_get_paned_key = rb_daap_source_get_paned_key;
-	library_source_class->impl_has_first_added_column = (RBLibrarySourceFeatureFunc) rb_false_function;
-	library_source_class->impl_has_drop_support = (RBLibrarySourceFeatureFunc) rb_false_function;
+	browser_source_class->impl_get_paned_key = rb_daap_source_get_paned_key;
+	browser_source_class->impl_has_first_added_column = (RBBrowserSourceFeatureFunc) rb_false_function;
+	browser_source_class->impl_has_drop_support = (RBBrowserSourceFeatureFunc) rb_false_function;
 
 
 	g_object_class_install_property (object_class,
@@ -252,7 +256,8 @@ rb_daap_source_get_property (GObject *object,
 }
 
 static GdkPixbuf *
-rb_daap_get_icon (void)
+rb_daap_get_icon (gboolean password_protected,
+		  gboolean connected)
 {
 	GdkPixbuf *icon;
 	GtkIconTheme *theme;
@@ -260,37 +265,51 @@ rb_daap_get_icon (void)
 
 	theme = gtk_icon_theme_get_default ();
 	gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
-	icon = gtk_icon_theme_load_icon (theme, "gnome-fs-network", size, 0, NULL);
+
+	if (! password_protected) {
+		icon = g_object_ref (daap_share_pixbuf);
+	} else if (connected) {
+		icon = g_object_ref (daap_share_pixbuf);
+	} else {
+		icon = g_object_ref (daap_share_pixbuf_locked);
+	}
 
 	return icon;
 }
 
 static RBSource *
 rb_daap_source_new (RBShell *shell,
-		    const gchar *service_name,
-		    const gchar *name,
-		    const gchar *host,
+		    const char *service_name,
+		    const char *name,
+		    const char *host,
 		    guint port,
 		    gboolean password_protected)
 {
 	RBSource *source;
 	RhythmDBEntryType type;
+	GdkPixbuf *icon;
 
 	type = rhythmdb_entry_daap_type_new ();
 
+	icon = rb_daap_get_icon (password_protected, FALSE);
+
 	source = RB_SOURCE (g_object_new (RB_TYPE_DAAP_SOURCE,
 					  "service-name", service_name,
 					  "name", name,
 					  "host", host,
 					  "port", port,
 					  "entry-type", type,
-					  "icon", rb_daap_get_icon (),
+					  "icon", icon,
 					  "shell", shell,
 					  "visibility", TRUE,
 					  "sorting-key", CONF_STATE_SORTING,
 					  "password-protected", password_protected,
 					  NULL));
 
+	if (icon != NULL) {
+		g_object_unref (icon);
+	}
+
 	rb_shell_register_entry_type_for_source (shell, source,
 						 type);
 
@@ -299,165 +318,135 @@ rb_daap_source_new (RBShell *shell,
 
 
 static RBSource *
-find_source_by_service_name (const gchar *service_name)
+find_source_by_service_name (const char *service_name)
 {
-	GSList *l;
+	RBSource *source;
 
-	for (l = sources; l != NULL; l = l->next) {
-		RBSource *source = l->data;
+	source = g_hash_table_lookup (source_lookup, service_name);
 
-		if (strcmp (service_name, RB_DAAP_SOURCE (source)->priv->service_name) == 0) {
-			return source;
-		}
-	}
-
-	return NULL;
+	return source;
 }
 
 static void
-resolve_cb (RBDAAPmDNSResolver resolver,
-	    RBDAAPmDNSResolverStatus status,
-	    const gchar *service_name,
-	    gchar *name,
-	    gchar *host,
-	    guint port,
-	    gboolean password_protected,
-	    RBShell *shell)
+mdns_service_added (RBDaapMdnsBrowser *browser,
+		    const char        *service_name,
+		    const char        *name,
+		    const char        *host,
+		    guint              port,
+		    gboolean           password_protected,
+		    RBShell           *shell)
 {
-	if (status == RB_DAAP_MDNS_RESOLVER_FOUND) {
-		RBSource *source = find_source_by_service_name (service_name);
+	RBSource *source;
 
-		if (source == NULL) {
-			source = rb_daap_source_new (shell, service_name, name, host, port, password_protected);
-			sources = g_slist_prepend (sources, source);
-			rb_shell_append_source (shell, source, NULL);
-		} else {
-			g_object_set (G_OBJECT (source),
-				      "name", name,
-				      "host", host,
-				      "port", port,
-				      "password-protected", password_protected,
-				      NULL);
-		}
-	} else if (status == RB_DAAP_MDNS_RESOLVER_TIMEOUT) {
-		g_warning ("Unable to resolve %s", service_name);
-	}
-}
+#if 0
+	g_message ("New service: %s name=%s host=%s port=%u password=%d",
+		   service_name, name, host, port, password_protected);
+#endif
 
+	source = find_source_by_service_name (service_name);
 
-static void
-browse_cb (RBDAAPmDNSBrowser b,
-	   RBDAAPmDNSBrowserStatus status,
-	   const gchar *service_name,
-	   RBShell *shell)
-{
-	if (status == RB_DAAP_MDNS_BROWSER_ADD_SERVICE) {
-		gboolean ret;
-		RBDAAPmDNSResolver *resolver;
-
-		if (find_source_by_service_name (service_name)) {
-			rb_debug ("Ignoring duplicate DAAP source");
-			return;
-		}
-
-		rb_debug ("New DAAP (music sharing) source '%s' discovered", service_name);
-
-		/* the resolver takes care of ignoring our own shares,
-		 * if this is us, the callback won't fire
-		 */
-		resolver = g_new0 (RBDAAPmDNSResolver, 1);
-		g_hash_table_insert (service_name_to_resolver, g_strdup (service_name), resolver);
-		ret = rb_daap_mdns_resolve (resolver,
-					    service_name,
-					    (RBDAAPmDNSResolverCallback) resolve_cb,
-					    shell);
-		if (!ret) {
-			g_warning ("could not start resolving host");
-		}
-
-	} else if (status == RB_DAAP_MDNS_BROWSER_REMOVE_SERVICE) {
-		RBSource *source = find_source_by_service_name (service_name);
-
-		rb_debug ("DAAP source '%s' went away", service_name);
-		if (source == NULL) {
-			/* if this happens, its because the user's own share
-			 * went away.  since that one doesnt resolve,
-			 * it doesnt get added to the sources list
-			 * it does have a resolver tho, so we should remove
-			 * that.
-			 */
-			g_hash_table_remove (service_name_to_resolver, service_name);
-
-			return;
-		}
-
-		g_hash_table_remove (service_name_to_resolver, service_name);
-		sources = g_slist_remove (sources, source);
-
-		rb_daap_source_disconnect (source);
-		rb_source_delete_thyself (source);
-		/* unref is done via gtk in rb_shell_source_delete_cb at
-		 * gtk_notebook_remove_page */
+	if (source == NULL) {
+		source = rb_daap_source_new (shell, service_name, name, host, port, password_protected);
+		g_hash_table_insert (source_lookup, g_strdup (service_name), source);
+		rb_shell_append_source (shell, source, NULL);
+	} else {
+		g_object_set (G_OBJECT (source),
+			      "name", name,
+			      "host", host,
+			      "port", port,
+			      "password-protected", password_protected,
+			      NULL);
 	}
 }
 
 static void
-stop_resolver (RBDAAPmDNSResolver *resolver)
+mdns_service_removed (RBDaapMdnsBrowser *browser,
+		      const char        *service_name,
+		      RBShell           *shell)
 {
-	rb_daap_mdns_resolve_cancel (*resolver);
-	g_free (resolver);
+	RBSource *source;
 
-	resolver = NULL;
+	source = find_source_by_service_name (service_name);
+
+	rb_debug ("DAAP source '%s' went away", service_name);
+	if (source == NULL) {
+		return;
+	}
+
+	g_hash_table_remove (source_lookup, service_name);
 }
 
+static void
+remove_source (RBSource *source)
+{
+	rb_daap_source_disconnect (source);
+	rb_source_delete_thyself (source);
+}
 
 static void
 start_browsing (RBShell *shell)
 {
-	if (service_name_to_resolver != NULL)
+	GError *error;
+
+	if (mdns_browser != NULL) {
 		return;
+	}
 
-	gboolean ret = rb_daap_mdns_browse (&browser,
-					   (RBDAAPmDNSBrowserCallback) browse_cb,
-					   shell);
-
-	if (!ret) {
+	mdns_browser = rb_daap_mdns_browser_new ();
+	if (mdns_browser == NULL) {
 		g_warning ("Unable to start mDNS browsing");
 		return;
 	}
 
-	service_name_to_resolver = g_hash_table_new_full ((GHashFunc)g_str_hash,
-		       				 	  (GEqualFunc)g_str_equal, 
-							  (GDestroyNotify)g_free, 
-							  (GDestroyNotify)stop_resolver);
+	g_signal_connect_object (mdns_browser,
+				 "service-added",
+				 G_CALLBACK (mdns_service_added),
+				 shell,
+				 0);
+	g_signal_connect_object (mdns_browser,
+				 "service-removed",
+				 G_CALLBACK (mdns_service_removed),
+				 shell,
+				 0);
+
+	error = NULL;
+	rb_daap_mdns_browser_start (mdns_browser, &error);
+	if (error != NULL) {
+		g_warning ("Unable to start mDNS browsing: %s", error->message);
+		g_error_free (error);
+	}
+
+	source_lookup = g_hash_table_new_full ((GHashFunc)g_str_hash,
+					       (GEqualFunc)g_str_equal, 
+					       (GDestroyNotify)g_free, 
+					       (GDestroyNotify)remove_source);
 
 }
 
 static void
 stop_browsing (RBShell *shell)
 {
-	GSList *l;
+	GError *error;
 
-	if (service_name_to_resolver == NULL)
+	if (mdns_browser == NULL) {
 		return;
-
-	g_hash_table_destroy (service_name_to_resolver);
-	service_name_to_resolver = NULL;
-
-	for (l = sources; l != NULL; l = l->next) {
-		RBSource *source = l->data;
-
-		rb_daap_source_disconnect (source);
-		rb_source_delete_thyself (source);
 	}
 
-	g_slist_free (sources);
-	sources = NULL;
+	g_hash_table_destroy (source_lookup);
+	source_lookup = NULL;
 
-	if (browser) {
-		rb_daap_mdns_browse_cancel (browser);
-		browser = 0;
+	g_signal_handlers_disconnect_by_func (mdns_browser, mdns_service_added, shell);
+	g_signal_handlers_disconnect_by_func (mdns_browser, mdns_service_removed, shell);
+
+	error = NULL;
+	rb_daap_mdns_browser_stop (mdns_browser, &error);
+	if (error != NULL) {
+		g_warning ("Unable to stop mDNS browsing: %s", error->message);
+		g_error_free (error);
 	}
+
+	g_object_unref (mdns_browser);
+	mdns_browser = NULL;
 }
 
 static void
@@ -475,6 +464,92 @@ enable_browsing_changed_cb (GConfClient *client,
 	}
 }
 
+static GdkPixbuf *
+composite_icons (const GdkPixbuf *src1,
+		 const GdkPixbuf *src2)
+{
+	GdkPixbuf *dest;
+	GdkPixbuf *scaled;
+	gint       w1, w2, h1, h2;
+	gint       dest_x, dest_y;
+	gboolean   do_scale;
+
+	if (! src1) {
+		return NULL;
+	}
+
+	dest = gdk_pixbuf_copy (src1);
+
+	if (! src2) {
+		return dest;
+	}
+
+	w1 = gdk_pixbuf_get_width (src1);
+	h1 = gdk_pixbuf_get_height (src1);
+	w2 = gdk_pixbuf_get_width (src2);
+	h2 = gdk_pixbuf_get_height (src2);
+
+	do_scale = ((float)w1 * 0.8) < w2;
+
+	/* scale the emblem down if it will obscure the entire bottom image */
+	if (do_scale) {
+		scaled = gdk_pixbuf_scale_simple (src2, w1 / 2, h1 / 2, GDK_INTERP_BILINEAR);
+	} else {
+		scaled = (GdkPixbuf *)src2;
+	}
+
+	w2 = gdk_pixbuf_get_width (scaled);
+	h2 = gdk_pixbuf_get_height (scaled);
+
+	dest_x = w1 - w2;
+	dest_y = h1 - h2;
+
+	gdk_pixbuf_composite (scaled, dest,
+			      dest_x, dest_y,
+			      w2, h2,
+			      dest_x, dest_y,
+			      1.0, 1.0,
+			      GDK_INTERP_BILINEAR, 0xFF);
+
+	if (do_scale) {
+		g_object_unref (scaled);
+	}
+
+	return dest;
+}
+
+static void
+create_pixbufs (void)
+{
+	GdkPixbuf    *emblem;
+	GtkIconTheme *theme;
+	gint          size;
+
+	theme = gtk_icon_theme_get_default ();
+
+	gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
+	daap_share_pixbuf = gtk_icon_theme_load_icon (theme, "gnome-fs-network", size, 0, NULL);
+
+	gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &size, NULL);
+	emblem = gtk_icon_theme_load_icon (theme, "stock_lock", size, 0, NULL);
+
+	daap_share_pixbuf_locked = composite_icons (daap_share_pixbuf, emblem);
+}
+
+static void
+destroy_pixbufs (void)
+{
+	if (daap_share_pixbuf != NULL) {
+		g_object_unref (daap_share_pixbuf);
+	}
+	daap_share_pixbuf = NULL;
+
+	if (daap_share_pixbuf_locked != NULL) {
+		g_object_unref (daap_share_pixbuf_locked);
+	}
+	daap_share_pixbuf_locked = NULL;
+}
+
 RBSource *
 rb_daap_sources_init (RBShell *shell)
 {
@@ -500,13 +575,15 @@ rb_daap_sources_init (RBShell *shell)
 					    (GConfClientNotifyFunc) enable_browsing_changed_cb,
 					    shell);
 
+	create_pixbufs ();
+
 	return NULL;
 }
 
 void
 rb_daap_sources_shutdown (RBShell *shell)
 {
-	if (browser) {
+	if (mdns_browser) {
 		stop_browsing (shell);
 	}
 
@@ -515,9 +592,54 @@ rb_daap_sources_shutdown (RBShell *shell)
 		enable_browsing_notify_id = EEL_GCONF_UNDEFINED_CONNECTION;
 	}
 
+	destroy_pixbufs ();
+
 	g_object_unref (shell);
 }
 
+static char *
+connection_auth_cb (RBDAAPConnection *connection,
+		    const char       *name,
+		    RBSource         *source)
+{
+	char      *password;
+	GtkWindow *parent;
+
+	parent = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (source)));
+	password = rb_daap_password_dialog_new_run (parent, name);
+
+	return password;
+}
+
+static void
+connection_connected_cb (RBDAAPConnection *connection,
+			 RBDAAPSource     *source)
+{
+	GdkPixbuf *icon;
+
+	rb_debug ("DAAP connection connected");
+
+	icon = rb_daap_get_icon (source->priv->password_protected, TRUE);
+	g_object_set (source, "icon", icon, NULL);
+	if (icon != NULL) {
+		g_object_unref (icon);
+	}
+}
+
+static void
+connection_disconnected_cb (RBDAAPConnection *connection,
+			    RBDAAPSource     *source)
+{
+	GdkPixbuf *icon;
+
+	rb_debug ("DAAP connection disconnected");
+
+	icon = rb_daap_get_icon (source->priv->password_protected, FALSE);
+	g_object_set (source, "icon", icon, NULL);
+	if (icon != NULL) {
+		g_object_unref (icon);
+	}
+}
 
 static void
 rb_daap_source_activate (RBSource *source)
@@ -525,7 +647,7 @@ rb_daap_source_activate (RBSource *source)
 	RBDAAPSource *daap_source = RB_DAAP_SOURCE (source);
 	RBShell *shell = NULL;
 	RhythmDB *db = NULL;
-	gchar *name = NULL;
+	char *name = NULL;
 	RhythmDBEntryType type;
 
 	if (daap_source->priv->connection != NULL) 
@@ -549,11 +671,25 @@ rb_daap_source_activate (RBSource *source)
 					source);
 	g_object_unref (G_OBJECT (db));
 	g_object_unref (G_OBJECT (shell));
+
 	if (daap_source->priv->connection == NULL) {
 		/* XXX can this still happen? */
 		daap_source->priv->playlist_sources = NULL;
 		return;
 	}
+
+        g_signal_connect (daap_source->priv->connection,
+			  "authenticate",
+                          G_CALLBACK (connection_auth_cb),
+			  source);
+        g_signal_connect (daap_source->priv->connection,
+			  "connected",
+                          G_CALLBACK (connection_connected_cb),
+			  source);
+        g_signal_connect (daap_source->priv->connection,
+			  "disconnected",
+                          G_CALLBACK (connection_disconnected_cb),
+			  source);
 }
 
 static void
@@ -596,6 +732,9 @@ rb_daap_source_disconnect_cb (RBDAAPConnection *connection,
 			      RBSource *source)
 {
 	RBDAAPSource *daap_source = RB_DAAP_SOURCE (source);
+
+	rb_debug ("DAAP source disconnected");
+
 	g_object_unref (G_OBJECT (connection));
 	daap_source->priv->connection = NULL;
 	g_object_unref (source);
@@ -648,41 +787,61 @@ rb_daap_source_show_popup (RBSource *source)
 	return TRUE;
 }
 
-RBDAAPSource *
-rb_daap_source_find_for_uri (const gchar *uri)
+static gboolean
+source_host_find (const char *key,
+		  RBDAAPSource *source,
+		  const char *host)
 {
-	GSList *l;
-	gchar *ip;
-	gchar *s;
-	RBDAAPSource *found_source = NULL;
+	if (source == NULL || host == NULL) {
+		return FALSE;
+	}
 
-	ip = strdup (uri + 7); // daap://
+	if (strcmp (host, source->priv->host) == 0) {
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+RBDAAPSource *
+rb_daap_source_find_for_uri (const char *uri)
+{
+	char *ip;
+	char *s;
+	RBDAAPSource *source = NULL;
+
+	if (uri == NULL) {
+		return NULL;
+	}
+
+	ip = strdup (uri + 7); /* daap:// */
 	s = strchr (ip, ':');
 	*s = '\0';
 
-	for (l = sources; l != NULL; l = l->next) {
-		RBDAAPSource *source = l->data;
-
-		if (strcmp (ip, source->priv->host) == 0) {
-			found_source = source;
-			break;
-		}
-
-	}
+	source = (RBDAAPSource *)g_hash_table_find (source_lookup, (GHRFunc)source_host_find, ip);
 
 	g_free (ip);
-
-	return found_source;
+	
+	return source;
 }
 
-gchar *
+char *
 rb_daap_source_get_headers (RBDAAPSource *source,
-			    const gchar *uri,
+			    const char *uri,
 			    glong time,
 			    gint64 *bytes_out)
 {
 	gint64 bytes = 0;
 
+	if (bytes_out != NULL) {
+		*bytes_out = 0;
+	}
+
+	/* If there is no connection then bail */
+	if (source->priv->connection == NULL) {
+		return NULL;
+	}
+
 	if (time != 0) {
 		RhythmDB *db;
 		RBShell *shell;
@@ -698,24 +857,27 @@ rb_daap_source_get_headers (RBDAAPSource *source,
 		g_object_unref (G_OBJECT (db));
 		
 		bitrate = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_BITRATE); 
-		// bitrate is kilobits per second
-		// a kilobit is 128 bytes
+		/* bitrate is kilobits per second */
+		/* a kilobit is 128 bytes */
 		bytes = (time * bitrate) * 128; 
 	}
 	
-	*bytes_out = bytes;
+	if (bytes_out != NULL) {
+		*bytes_out = bytes;
+	}
+
 	return rb_daap_connection_get_headers (source->priv->connection, uri, bytes);
 }
 
 
-static const gchar * 
+static char * 
 rb_daap_source_get_browser_key (RBSource *source)
 {
-	return CONF_STATE_SHOW_BROWSER;
+	return g_strdup (CONF_STATE_SHOW_BROWSER);
 }
 
-static const gchar * 
-rb_daap_source_get_paned_key (RBLibrarySource *source)
+static const char * 
+rb_daap_source_get_paned_key (RBBrowserSource *source)
 {
 	return CONF_STATE_PANED_POSITION;
 }
