@@ -20,9 +20,6 @@
  */
 
 #include <config.h>
-#if defined(HAVE_HAL_0_5) || defined(HAVE_HAL_0_2)
-#define HAVE_HAL 1
-#endif
 
 
 #include <gtk/gtktreeview.h>
@@ -39,7 +36,7 @@
 #include "eel-gconf-extensions.h"
 #include "rb-ipod-source.h"
 #include "rb-debug.h"
-#include "rb-playlist-source.h"
+#include "rb-static-playlist-source.h"
 #include "rb-util.h"
 #include "rhythmdb.h"
 
@@ -96,12 +93,15 @@ static GObject *
 rb_ipod_source_constructor (GType type, guint n_construct_properties,
 			       GObjectConstructParam *construct_properties)
 {
-	GObjectClass *klass, *parent_class; 
-	RBiPodSource *source; 
+	RBiPodSource *source;
+	RBEntryView *songs;
 
-	klass = G_OBJECT_CLASS (g_type_class_peek (type));
-	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
-	source = RB_IPOD_SOURCE (parent_class->constructor (type, n_construct_properties, construct_properties));
+	source = RB_IPOD_SOURCE (G_OBJECT_CLASS (rb_ipod_source_parent_class)->
+			constructor (type, n_construct_properties, construct_properties));
+
+	songs = rb_source_get_entry_view (RB_SOURCE (source));
+	rb_entry_view_append_column (songs, RB_ENTRY_VIEW_COL_RATING, FALSE);
+	rb_entry_view_append_column (songs, RB_ENTRY_VIEW_COL_LAST_PLAYED, FALSE);
 
 	rb_ipod_load_songs (source);
 
@@ -137,7 +137,6 @@ rb_ipod_source_new (RBShell *shell, GnomeVFSVolume *volume)
 	entry_type =  rhythmdb_entry_register_type ();
 
 	source = RB_IPOD_SOURCE (g_object_new (RB_TYPE_IPOD_SOURCE,
-					  "name", _("iPod"),
 					  "entry-type", entry_type,
 					  "volume", volume,
 					  "shell", shell,
@@ -194,13 +193,10 @@ add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
 		      "entry-type", &entry_type,
 		      NULL);
 
-	playlist_source = RB_SOURCE (g_object_new (RB_TYPE_PLAYLIST_SOURCE,
-						   "name", playlist->name,
-						   "shell", shell,
-						   "visibility", TRUE,
-						   "is-local", FALSE,
-						   "entry-type", entry_type,
-						   NULL));
+	playlist_source = rb_static_playlist_source_new (shell, 
+							 playlist->name, 
+							 FALSE,
+							 entry_type);
 
 	for (it = playlist->members; it != NULL; it = it->next) {
 		Itdb_Track *song;
@@ -209,8 +205,8 @@ add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
 		song = (Itdb_Track *)it->data;
  		filename = ipod_path_to_uri (priv->ipod_mount_path, 
  					    song->ipod_path);
-		rb_playlist_source_add_location (RB_PLAYLIST_SOURCE (playlist_source),
-						 filename);
+		rb_static_playlist_source_add_location (RB_STATIC_PLAYLIST_SOURCE (playlist_source),
+							filename, -1);
 		g_free (filename);
 	}
 
@@ -270,9 +266,15 @@ load_ipod_db_idle_cb (RBiPodSource *source)
  					    song->ipod_path);
   		entry = rhythmdb_entry_new (RHYTHMDB (db), entry_type,
  					    pc_path);
- 		g_free (pc_path);
-  
+
+		if (entry == NULL) {
+			rb_debug ("cannot create entry %s", pc_path);
+ 			g_free (pc_path);
+			continue;
+		}
+		
 		rb_debug ("Adding %s from iPod", pc_path);
+ 		g_free (pc_path);
 
 		/* Set track number */
 		if (song->track_nr != 0) {
@@ -328,6 +330,61 @@ load_ipod_db_idle_cb (RBiPodSource *source)
 						       &value);
 			g_value_unset (&value);
 		}
+
+		/* Set playcount */
+		if (song->playcount != 0) {
+			GValue value = {0, };
+			g_value_init (&value, G_TYPE_ULONG);
+			g_value_set_ulong (&value, song->playcount);
+			rhythmdb_entry_set_uninserted (RHYTHMDB (db), entry,
+						       RHYTHMDB_PROP_PLAY_COUNT,
+						       &value);
+			g_value_unset (&value);
+		}
+
+		/* Set year */
+		if (song->year != 0) {
+			GDate *date = NULL;
+			GType type;
+			GValue value = {0, };
+			
+			date = g_date_new_dmy (1, G_DATE_JANUARY, song->year);
+
+			type = rhythmdb_get_property_type (RHYTHMDB(db),
+							    RHYTHMDB_PROP_DATE);
+			
+			g_value_init (&value, type);
+			g_value_set_ulong (&value, (date ? g_date_get_julian (date) : 0));
+			
+			rhythmdb_entry_set_uninserted (RHYTHMDB (db), entry,
+						       RHYTHMDB_PROP_DATE,
+						       &value);
+			g_value_unset (&value);
+			if (date)
+				g_date_free (date);
+		}
+
+		/* Set rating */
+		if (song->rating != 0) {
+			GValue value = {0, };
+			g_value_init (&value, G_TYPE_DOUBLE);
+			g_value_set_double (&value, song->rating/20.0);
+			rhythmdb_entry_set_uninserted (RHYTHMDB (db), entry,
+						       RHYTHMDB_PROP_RATING,
+						       &value);
+			g_value_unset (&value);
+		}
+		
+		/* Set last played */
+		if (song->time_played != 0) {
+			GValue value = {0, };
+			g_value_init (&value, G_TYPE_ULONG);
+			g_value_set_ulong (&value, itdb_time_mac_to_host (song->time_played));
+			rhythmdb_entry_set_uninserted (RHYTHMDB (db), entry,
+						       RHYTHMDB_PROP_LAST_PLAYED,
+						       &value);
+			g_value_unset (&value);
+		}
 		
 		/* Set title */		
 		entry_set_string_prop (RHYTHMDB (db), entry, 
@@ -450,41 +507,51 @@ hal_udi_is_ipod (const char *udi)
 
 	result = FALSE;
 	dbus_error_init (&error);
+	
 	conn = NULL;
 	ctx = libhal_ctx_new ();
 	if (ctx == NULL) {
 		/* FIXME: should we return an error somehow so that we can 
 		 * fall back to a check for iTunesDB presence instead ?
 		 */
-		g_print ("Error: %s\n", error.message);
+		rb_debug ("cannot connect to HAL");
 		goto end;
 	}
 	conn = dbus_bus_get (DBUS_BUS_SYSTEM, &error);
-	if (conn == NULL) {
-		g_print ("Error: %s\n", error.message);
+	if (conn == NULL || dbus_error_is_set (&error))
 		goto end;
-	}
+
 	libhal_ctx_set_dbus_connection (ctx, conn);
-	if (!libhal_ctx_init (ctx, &error)) {
-		g_print ("Error: %s\n", error.message);
+	if (!libhal_ctx_init (ctx, &error) || dbus_error_is_set (&error))
 		goto end;
-	}
 
 	parent_udi = libhal_device_get_property_string (ctx, udi,
 			"info.parent", &error);
+	if (parent_udi == NULL || dbus_error_is_set (&error))
+		goto end;
+		
 	parent_name = libhal_device_get_property_string (ctx, parent_udi,
 			"storage.model", &error);
 	g_free (parent_udi);
-	if (parent_name != NULL && strcmp (parent_name, "iPod") == 0) {
+	if (parent_name == NULL || dbus_error_is_set (&error))
+		goto end;
+
+	if (strcmp (parent_name, "iPod") == 0)
 		result = TRUE;
-	}
 
 	g_free (parent_name);
 end:
+	if (dbus_error_is_set (&error)) {
+		rb_debug ("Error: %s\n", error.message);
+		dbus_error_free (&error);
+		dbus_error_init (&error);
+	}
+
 	if (ctx) {
 		libhal_ctx_shutdown (ctx, &error);
 		libhal_ctx_free(ctx);
 	}
+
 	dbus_error_free (&error);
 
 	return result;
