@@ -43,16 +43,10 @@
 #include "rb-preferences.h"
 #include "rb-dialog.h"
 #include "rb-station-properties-dialog.h"
+#include "rb-new-station-dialog.h"
 #include "rb-debug.h"
 #include "eel-gconf-extensions.h"
 
-typedef enum
-{
-	RB_IRADIO_QUERY_TYPE_ALL,
-	RB_IRADIO_QUERY_TYPE_GENRE,
-	RB_IRADIO_QUERY_TYPE_SEARCH,
-} RBIRadioQueryType;
-
 static void rb_iradio_source_class_init (RBIRadioSourceClass *klass);
 static void rb_iradio_source_init (RBIRadioSource *source);
 static GObject *rb_iradio_source_constructor (GType type, guint n_construct_properties,
@@ -68,6 +62,7 @@ static void rb_iradio_source_get_property (GObject *object,
 			                  GValue *value,
 			                  GParamSpec *pspec);
 static void rb_iradio_source_songs_show_popup_cb (RBEntryView *view,
+						  gboolean over_entry,
 						  RBIRadioSource *source);
 static void paned_size_allocate_cb (GtkWidget *widget,
 				    GtkAllocation *allocation,
@@ -87,18 +82,20 @@ static void genre_selected_cb (RBPropertyView *propview, const char *name,
 			       RBIRadioSource *iradio_source);
 static void genre_selection_reset_cb (RBPropertyView *propview, RBIRadioSource *iradio_source);
 static void rb_iradio_source_songs_view_sort_order_changed_cb (RBEntryView *view, RBIRadioSource *source);
+static char *guess_uri_scheme (const char *uri);
 
 /* source methods */
 static char *impl_get_status (RBSource *source);
-static const char *impl_get_browser_key (RBSource *source);
-static GdkPixbuf *impl_get_pixbuf (RBSource *source);
+static char *impl_get_browser_key (RBSource *source);
 static RBEntryView *impl_get_entry_view (RBSource *source);
 static void impl_search (RBSource *source, const char *text);
 static void impl_delete (RBSource *source);
 static void impl_song_properties (RBSource *source);
 static RBSourceEOFType impl_handle_eos (RBSource *asource);
 static gboolean impl_show_popup (RBSource *source);
-static void rb_iradio_source_do_query (RBIRadioSource *source, RBIRadioQueryType type);
+static GList *impl_get_ui_actions (RBSource *source);
+
+static void rb_iradio_source_do_query (RBIRadioSource *source);
 
 void rb_iradio_source_show_columns_changed_cb (GtkToggleButton *button,
 					     RBIRadioSource *source);
@@ -108,6 +105,8 @@ static void stations_view_drag_data_received_cb (GtkWidget *widget,
 						 GtkSelectionData *data,
 						 guint info, guint time,
 						 RBIRadioSource *source);
+static void rb_iradio_source_cmd_new_station (GtkAction *action,
+					      RBIRadioSource *source);
 
 #define CMD_PATH_SHOW_BROWSER "/commands/ShowBrowser"
 #define CMD_PATH_CURRENT_STATION "/commands/CurrentStation"
@@ -125,30 +124,32 @@ struct RBIRadioSourcePrivate
 	RhythmDB *db;
 
 	GtkWidget *vbox;
-
-	GdkPixbuf *pixbuf;
+	GtkWidget *paned;
+	GtkActionGroup *action_group;
 
 	RBPropertyView *genres;
 	RBEntryView *stations;
-
-	char *title, *song, *name;
+	gboolean setting_new_query;
 
 	gboolean initialized;
 
-	char *status;
-
-	GtkWidget *paned;
-
 	char *search_text;
 	char *selected_genre;
 
+	gboolean firstrun_done;
+	
 	RhythmDBEntryType entry_type;
-
-	RhythmDBQueryModel *all_query;
 };
 
 #define RB_IRADIO_SOURCE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_IRADIO_SOURCE, RBIRadioSourcePrivate))
 
+static GtkActionEntry rb_iradio_source_actions [] =
+{
+	{ "MusicNewInternetRadioStation", GTK_STOCK_NEW, N_("New Internet _Radio Station"), "<control>I",
+	  N_("Create a new Internet Radio station"),
+	  G_CALLBACK (rb_iradio_source_cmd_new_station) }
+};
+
 static const GtkTargetEntry stations_view_drag_types[] = {
 	{  "text/uri-list", 0, 0 },
 	{  "_NETSCAPE_URL", 0, 1 },
@@ -175,22 +176,22 @@ rb_iradio_source_class_init (RBIRadioSourceClass *klass)
 	object_class->set_property = rb_iradio_source_set_property;
 	object_class->get_property = rb_iradio_source_get_property;
 
-	source_class->impl_get_status  = impl_get_status;
 	source_class->impl_can_browse = (RBSourceFeatureFunc) rb_true_function;
-	source_class->impl_get_browser_key  = impl_get_browser_key;
-	source_class->impl_get_pixbuf  = impl_get_pixbuf;
-	source_class->impl_can_search = (RBSourceFeatureFunc) rb_true_function;
-	source_class->impl_search = impl_search;
-	source_class->impl_get_entry_view = impl_get_entry_view;
-	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_true_function;
-	source_class->impl_delete = impl_delete;
-	source_class->impl_song_properties = impl_song_properties;
-	source_class->impl_can_pause = (RBSourceFeatureFunc) rb_false_function;
-	source_class->impl_handle_eos = impl_handle_eos;
-	source_class->impl_try_playlist = (RBSourceFeatureFunc) rb_true_function;
-	source_class->impl_have_url = (RBSourceFeatureFunc) rb_true_function;
-	source_class->impl_show_popup = impl_show_popup;
 	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_false_function;
+	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_true_function;
+	source_class->impl_can_pause = (RBSourceFeatureFunc) rb_false_function;
+	source_class->impl_can_search = (RBSourceFeatureFunc) rb_true_function;
+	source_class->impl_delete = impl_delete;
+	source_class->impl_get_browser_key  = impl_get_browser_key;
+	source_class->impl_get_entry_view = impl_get_entry_view;
+	source_class->impl_get_status  = impl_get_status;
+	source_class->impl_get_ui_actions = impl_get_ui_actions;
+	source_class->impl_handle_eos = impl_handle_eos;
+	source_class->impl_have_url = (RBSourceFeatureFunc) rb_true_function;
+	source_class->impl_search = impl_search;
+	source_class->impl_show_popup = impl_show_popup;
+	source_class->impl_song_properties = impl_song_properties;
+	source_class->impl_try_playlist = (RBSourceFeatureFunc) rb_true_function;
 
 	g_object_class_install_property (object_class,
 					 PROP_ENTRY_TYPE,
@@ -209,6 +210,7 @@ static void
 rb_iradio_source_init (RBIRadioSource *source)
 {
 	gint size;
+	GdkPixbuf *pixbuf;
 
 	source->priv = RB_IRADIO_SOURCE_GET_PRIVATE (source);
 
@@ -216,15 +218,15 @@ rb_iradio_source_init (RBIRadioSource *source)
 
 	gtk_container_add (GTK_CONTAINER (source), source->priv->vbox);
 	
-	source->priv->song = g_strdup (_("Unknown"));
-	source->priv->name = g_strdup (_("Unknown"));
-	source->priv->title = g_strdup (_("Unknown"));
-
 	gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
-	source->priv->pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-							 "stock_channel",
-							 size,
-							 0, NULL);
+	pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+					   "stock_channel",
+					   size,
+					   0, NULL);
+	rb_source_set_pixbuf (RB_SOURCE (source), pixbuf);
+	if (pixbuf != NULL) {
+		g_object_unref (pixbuf);
+	}
 }
 
 static void
@@ -234,10 +236,6 @@ rb_iradio_source_dispose (GObject *object)
 
 	source = RB_IRADIO_SOURCE (object);
 
-	if (source->priv->all_query) {
-		g_object_unref (source->priv->all_query);
-		source->priv->all_query = NULL;
-	}
 	if (source->priv->db) {
 		g_object_unref (source->priv->db);
 		source->priv->db = NULL;
@@ -260,11 +258,6 @@ rb_iradio_source_finalize (GObject *object)
 
 	rb_debug ("finalizing iradio source");
 
-	g_free (source->priv->name);
-	g_free (source->priv->song);
-	g_free (source->priv->title);
-	g_free (source->priv->status);
-
 	G_OBJECT_CLASS (rb_iradio_source_parent_class)->finalize (object);
 }
 
@@ -289,17 +282,22 @@ rb_iradio_source_constructor (GType type, guint n_construct_properties,
 	shell_player = rb_shell_get_player (shell);
 	g_object_unref (G_OBJECT (shell));
 
+	source->priv->action_group = _rb_source_register_action_group (RB_SOURCE (source),
+								       "IRadioActions",
+								       rb_iradio_source_actions,
+								       G_N_ELEMENTS (rb_iradio_source_actions),
+								       source);
 
 	/* set up stations view */
 	source->priv->stations = rb_entry_view_new (source->priv->db, shell_player,
 						    CONF_STATE_IRADIO_SORTING,
 						    FALSE, FALSE);
-	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_TITLE);
-	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_GENRE);
-/* 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_QUALITY); */
-	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_RATING);
-/*	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_PLAY_COUNT);*/
-	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_LAST_PLAYED);
+	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_TITLE, TRUE);
+	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_GENRE, FALSE);
+/* 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_QUALITY, FALSE); */
+	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_RATING, FALSE);
+/*	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_PLAY_COUNT, FALSE);*/
+	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_LAST_PLAYED, FALSE);
 	g_signal_connect_object (G_OBJECT (source->priv->stations),
 				 "sort-order-changed",
 				 G_CALLBACK (rb_iradio_source_songs_view_sort_order_changed_cb),
@@ -352,12 +350,13 @@ rb_iradio_source_constructor (GType type, guint n_construct_properties,
 	eel_gconf_notification_add (CONF_STATE_IRADIO_DIR,
 				    (GConfClientNotifyFunc) rb_iradio_source_state_pref_changed,
 				    source);
+	source->priv->firstrun_done = eel_gconf_get_boolean (CONF_FIRST_TIME);
 	eel_gconf_notification_add (CONF_FIRST_TIME,
 				    (GConfClientNotifyFunc) rb_iradio_source_first_time_changed,
 				    source);
 	gtk_widget_show_all (GTK_WIDGET (source));
 
-	rb_iradio_source_do_query (source, RB_IRADIO_QUERY_TYPE_ALL);
+	rb_iradio_source_do_query (source);
 
 	return G_OBJECT (source);
 }
@@ -414,21 +413,50 @@ rb_iradio_source_new (RBShell *shell)
 	return source;
 }
 
+static char *
+guess_uri_scheme (const char *uri)
+{
+	const char *scheme;
+	
+	/* if the URI has no scheme, it might be an absolute path, or it might be
+	 * host:port for HTTP.
+	 */
+	scheme = strstr (uri, "://");
+	if (scheme == NULL) {
+		if (uri[0] == '/') {
+			return g_strdup_printf ("file://%s", uri);
+		} else {
+			return g_strdup_printf ("http://%s", uri);
+		}
+	}
+
+	return NULL;
+}
+
+
 void
 rb_iradio_source_add_station (RBIRadioSource *source,
 			      const char *uri, const char *title, const char *genre)
 {
 	RhythmDBEntry *entry;
 	GValue val = { 0, };
+	char *real_uri = NULL;
+
+	real_uri = guess_uri_scheme (uri);
+	if (real_uri)
+		uri = real_uri;
 
 	entry = rhythmdb_entry_lookup_by_location (source->priv->db, uri);
 	if (entry) {
 		rb_debug ("uri %s already in db", uri); 
+		g_free (real_uri);
 		return;
 	}
 	entry = rhythmdb_entry_new (source->priv->db, RHYTHMDB_ENTRY_TYPE_IRADIO_STATION, uri);
-	if (entry == NULL)
+	if (entry == NULL) {
+		g_free (real_uri);
 		return;
+	}
 
 	g_value_init (&val, G_TYPE_STRING);
 	if (title)
@@ -447,19 +475,13 @@ rb_iradio_source_add_station (RBIRadioSource *source,
 	g_value_unset (&val);
 	
 	g_value_init (&val, G_TYPE_DOUBLE);
-	g_value_set_double (&val, 2.5);
+	g_value_set_double (&val, 0.0);
 	rhythmdb_entry_set_uninserted (source->priv->db, entry, RHYTHMDB_PROP_RATING, &val);
 	g_value_unset (&val);
 	
 	rhythmdb_commit (source->priv->db);
-}
 
-static GdkPixbuf *
-impl_get_pixbuf (RBSource *asource)
-{
-	RBIRadioSource *source = RB_IRADIO_SOURCE (asource);
-
-	return source->priv->pixbuf;
+	g_free (real_uri);
 }
 
 static void
@@ -482,7 +504,7 @@ impl_search (RBSource *asource, const char *search_text)
 
 	g_free (source->priv->search_text);
 	source->priv->search_text = g_strdup (search_text);
-	rb_iradio_source_do_query (source, RB_IRADIO_QUERY_TYPE_SEARCH);
+	rb_iradio_source_do_query (source);
 
 	rb_source_notify_filter_changed (RB_SOURCE (source));
 }
@@ -550,10 +572,10 @@ impl_get_status (RBSource *asource)
 	return ret;
 }
 
-static const char *
+static char *
 impl_get_browser_key (RBSource *asource)
 {
-	return CONF_STATE_SHOW_BROWSER;
+	return g_strdup (CONF_STATE_SHOW_BROWSER);
 }
 
 static void
@@ -623,21 +645,28 @@ rb_iradio_source_songs_view_sort_order_changed_cb (RBEntryView *view,
 
 static void
 rb_iradio_source_songs_show_popup_cb (RBEntryView *view,
+				      gboolean over_entry,
 				      RBIRadioSource *source)
 {
 	if (G_OBJECT (source) == NULL) {
 		return;
 	}
-	_rb_source_show_popup (RB_SOURCE (source), "/IRadioViewPopup");
+	if (over_entry)
+		_rb_source_show_popup (RB_SOURCE (source), "/IRadioViewPopup");
+	else
+		_rb_source_show_popup (RB_SOURCE (source), "/IRadioSourcePopup");
 }
 
 static void
 genre_selected_cb (RBPropertyView *propview, const char *name,
 		   RBIRadioSource *iradio_source)
 {
+	if (iradio_source->priv->setting_new_query)
+		return;
+
 	g_free (iradio_source->priv->selected_genre);
 	iradio_source->priv->selected_genre = g_strdup (name);
-	rb_iradio_source_do_query (iradio_source, RB_IRADIO_QUERY_TYPE_GENRE);
+	rb_iradio_source_do_query (iradio_source);
 
 	rb_source_notify_filter_changed (RB_SOURCE (iradio_source));
 }
@@ -645,10 +674,13 @@ genre_selected_cb (RBPropertyView *propview, const char *name,
 static void
 genre_selection_reset_cb (RBPropertyView *propview, RBIRadioSource *iradio_source)
 {
+	if (iradio_source->priv->setting_new_query)
+		return;
+	
 	g_free (iradio_source->priv->selected_genre);
 	iradio_source->priv->selected_genre = NULL;
 
-	rb_iradio_source_do_query (iradio_source, RB_IRADIO_QUERY_TYPE_GENRE);
+	rb_iradio_source_do_query (iradio_source);
 
 	rb_source_notify_filter_changed (RB_SOURCE (iradio_source));
 }
@@ -672,77 +704,105 @@ rb_iradio_source_show_browser (RBIRadioSource *source,
 }
 
 static void
-rb_iradio_source_do_query (RBIRadioSource *source, RBIRadioQueryType qtype)
+rb_iradio_source_do_query (RBIRadioSource *source)
 {
 	RhythmDBEntryType entry_type;
-	RhythmDBQueryModel *query_model;
+	RhythmDBQueryModel *genre_query_model = NULL;
+	RhythmDBQueryModel *station_query_model = NULL;
 	RhythmDBPropertyModel *genre_model;
-	GtkTreeModel *model;
 	GPtrArray *query;
 
-	if ((qtype == RB_IRADIO_QUERY_TYPE_ALL) && (source->priv->all_query)) {
-		rb_debug ("using cached query");
-		rb_entry_view_set_model (source->priv->stations, RHYTHMDB_QUERY_MODEL (source->priv->all_query));
-		return;
-	}
+	/* don't update the selection while we're rebuilding the query */
+	source->priv->setting_new_query = TRUE;
 
+	/* construct and run the query for the search box.
+	 * this is used as the model for the genre view.
+	 */
+	
 	g_object_get (G_OBJECT (source), "entry-type", &entry_type, NULL);
 	query = rhythmdb_query_parse (source->priv->db,
 				      RHYTHMDB_QUERY_PROP_EQUALS,
 				      RHYTHMDB_PROP_TYPE,
 				      entry_type,
 				      RHYTHMDB_QUERY_END);
-	query_model = rhythmdb_query_model_new_empty (source->priv->db);
+	genre_query_model = rhythmdb_query_model_new_empty (source->priv->db);
 
-	if (qtype == RB_IRADIO_QUERY_TYPE_ALL) {
-		rb_debug ("building new all query");
-		rb_property_view_reset (source->priv->genres);
-		g_free (source->priv->selected_genre);
-		source->priv->selected_genre = NULL;
-
-		genre_model = rb_property_view_get_model (source->priv->genres);
-		g_object_set (G_OBJECT (genre_model), "query-model", query_model, NULL);
-		g_object_unref (G_OBJECT (genre_model));
-
-		source->priv->all_query = query_model;
-	} else {
-		if (source->priv->search_text) {
-			GPtrArray *subquery = rhythmdb_query_parse (source->priv->db,
-								    RHYTHMDB_QUERY_PROP_LIKE,
-								    RHYTHMDB_PROP_GENRE_FOLDED,
-								    source->priv->search_text,
-								    RHYTHMDB_QUERY_DISJUNCTION,
-								    RHYTHMDB_QUERY_PROP_LIKE,
-								    RHYTHMDB_PROP_TITLE_FOLDED,
-								    source->priv->search_text,
-								    RHYTHMDB_QUERY_END);
-			rhythmdb_query_append (source->priv->db,
-					       query,
-					       RHYTHMDB_QUERY_SUBQUERY,
-					       subquery,
-					       RHYTHMDB_QUERY_END);
-		}
-
-
-		if (source->priv->selected_genre)
-			rhythmdb_query_append (source->priv->db,
-					       query,
-					       RHYTHMDB_QUERY_PROP_EQUALS,
-					       RHYTHMDB_PROP_GENRE,
-					       source->priv->selected_genre,
-					       RHYTHMDB_QUERY_END);
+	if (source->priv->search_text) {
+		GPtrArray *subquery = rhythmdb_query_parse (source->priv->db,
+							    RHYTHMDB_QUERY_PROP_LIKE,
+							    RHYTHMDB_PROP_GENRE_FOLDED,
+							    source->priv->search_text,
+							    RHYTHMDB_QUERY_DISJUNCTION,
+							    RHYTHMDB_QUERY_PROP_LIKE,
+							    RHYTHMDB_PROP_TITLE_FOLDED,
+							    source->priv->search_text,
+							    RHYTHMDB_QUERY_END);
+		rb_debug ("searching for \"%s\"", source->priv->search_text);
+		rhythmdb_query_append (source->priv->db,
+				       query,
+				       RHYTHMDB_QUERY_SUBQUERY,
+				       subquery,
+				       RHYTHMDB_QUERY_END);
 	}
 
+	genre_model = rb_property_view_get_model (source->priv->genres);
+	g_object_set (G_OBJECT (genre_model), "query-model", genre_query_model, NULL);
 
-	model = GTK_TREE_MODEL (query_model);
-	
-	rb_entry_view_set_model (source->priv->stations, RHYTHMDB_QUERY_MODEL (query_model));
-	g_object_set (G_OBJECT (source), "query-model", query_model, NULL);
-
-	rhythmdb_do_full_query_parsed (source->priv->db, model, query);
+	rhythmdb_do_full_query_parsed (source->priv->db, 
+				       RHYTHMDB_QUERY_RESULTS (genre_query_model), 
+				       query);
 
 	rhythmdb_query_free (query);
+	query = NULL;
+	
+	/* check the selected genre is still available, and if not, select 'all' */
+	if (source->priv->selected_genre != NULL) {
+		GList *sel = NULL;
+		if (!rhythmdb_property_model_iter_from_string (genre_model,
+							       source->priv->selected_genre,
+							       NULL)) {
+			g_free (source->priv->selected_genre);
+			source->priv->selected_genre = NULL;
+		}
 
+		sel = g_list_prepend (sel, source->priv->selected_genre);
+		rb_property_view_set_selection (source->priv->genres, sel);
+		g_list_free (sel);
+	} 
+	g_object_unref (G_OBJECT (genre_model));
+
+	/* if a genre is selected, construct a new query for it, and create
+	 * a new model based on the search box query model.  otherwise, just
+	 * reuse the search box query model.
+	 */
+	
+	if (source->priv->selected_genre != NULL) {
+		rb_debug ("matching on genre \"%s\"", source->priv->selected_genre);
+		station_query_model = rhythmdb_query_model_new_empty (source->priv->db);
+		query = rhythmdb_query_parse (source->priv->db,
+					      RHYTHMDB_QUERY_PROP_EQUALS,
+					      RHYTHMDB_PROP_GENRE,
+					      source->priv->selected_genre,
+					      RHYTHMDB_QUERY_END);
+
+		g_object_set (station_query_model, 
+			      "query", query,
+			      "base-model", genre_query_model,
+			      NULL);
+		rhythmdb_query_free (query);
+		query = NULL;
+	} else {
+		station_query_model = genre_query_model;
+	}
+
+	rb_entry_view_set_model (source->priv->stations, station_query_model);
+	g_object_set (G_OBJECT (source), "query-model", station_query_model, NULL);
+
+	g_object_unref (G_OBJECT (genre_query_model));
+	if (station_query_model != genre_query_model)
+		g_object_unref (G_OBJECT (station_query_model));
+
+	source->priv->setting_new_query = FALSE;
 }
 
 static void
@@ -752,16 +812,25 @@ handle_playlist_entry_cb (TotemPlParser *playlist, const char *uri, const char *
 	rb_iradio_source_add_station (source, uri, title, genre);
 }
 
+
 void
 rb_iradio_source_add_from_playlist (RBIRadioSource *source,
 				    const char     *uri)
 {
 	TotemPlParser *parser = totem_pl_parser_new ();
+	char *real_uri;
+
+	real_uri = guess_uri_scheme (uri);
+	if (real_uri)
+		uri = real_uri;
 
 	g_signal_connect_object (G_OBJECT (parser), "entry",
 				 G_CALLBACK (handle_playlist_entry_cb),
 				 source, 0);
-	switch (totem_pl_parser_parse (parser, uri, FALSE)) {
+	if (g_object_class_find_property (G_OBJECT_GET_CLASS (parser), "recurse"))
+		g_object_set (G_OBJECT (parser), "recurse", FALSE, NULL);
+
+	switch (totem_pl_parser_parse (parser, uri, TRUE)) {
 	case TOTEM_PL_PARSER_RESULT_UNHANDLED:
 	case TOTEM_PL_PARSER_RESULT_IGNORED:
 		/* maybe it's the actual stream URL, then */
@@ -773,6 +842,7 @@ rb_iradio_source_add_from_playlist (RBIRadioSource *source,
 		break;
 	}
 	g_object_unref (G_OBJECT (parser));
+	g_free (real_uri);
 }
 
 static void
@@ -781,11 +851,16 @@ rb_iradio_source_first_time_changed (GConfClient *client,
 				     GConfEntry *entry,
 				     RBIRadioSource *source)
 {
-	char *uri = gnome_vfs_get_uri_from_local_path (rb_file ("iradio-initial.pls"));
+	char *uri;
 
+	if (source->priv->firstrun_done || !gconf_value_get_bool (entry->value))
+		return;
+
+	uri = gnome_vfs_get_uri_from_local_path (rb_file ("iradio-initial.pls"));
 	rb_iradio_source_add_from_playlist (source, uri);
-
 	g_free (uri);
+
+	source->priv->firstrun_done = TRUE;
 }
 
 static void
@@ -851,3 +926,39 @@ impl_show_popup (RBSource *source)
 	return TRUE;
 }
 
+static GList*
+impl_get_ui_actions (RBSource *source)
+{
+	GList *actions = NULL;
+
+	actions = g_list_prepend (actions, g_strdup ("MusicNewInternetRadioStation"));
+
+	return actions;
+}
+
+static void
+new_station_location_added (RBNewStationDialog *dialog,
+			    const char         *uri,
+			    RBIRadioSource     *source)
+{
+	rb_iradio_source_add_from_playlist (source, uri);
+}
+
+static void
+rb_iradio_source_cmd_new_station (GtkAction *action,
+				  RBIRadioSource *source)
+{
+	GtkWidget *dialog;
+
+	rb_debug ("Got new station command");
+
+	dialog = rb_new_station_dialog_new (source->priv->stations);
+	g_signal_connect_object (dialog, "location-added",
+				 G_CALLBACK (new_station_location_added),
+				 source, 0);
+
+	gtk_dialog_run (GTK_DIALOG (dialog));
+
+	gtk_widget_destroy (dialog);
+}
+
