@@ -26,7 +26,7 @@
 #include <gtk/gtkactiongroup.h>
 
 #include "rb-shell.h"
-#include "rb-source.h"
+#include "rb-browser-source.h"
 #include "rhythmdb.h"
 
 G_BEGIN_DECLS
@@ -38,32 +38,27 @@ G_BEGIN_DECLS
 #define RB_IS_LIBRARY_SOURCE_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), RB_TYPE_LIBRARY_SOURCE))
 #define RB_LIBRARY_SOURCE_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), RB_TYPE_LIBRARY_SOURCE, RBLibrarySourceClass))
 
+
 typedef struct RBLibrarySourcePrivate RBLibrarySourcePrivate;
 
 typedef struct
 {
-	RBSource parent;
+	RBBrowserSource parent;
 
 	RBLibrarySourcePrivate *priv;
 } RBLibrarySource;
 
 typedef struct
 {
-	RBSourceClass parent;
-
-	const char *	(*impl_get_paned_key)		(RBLibrarySource *source);
+	RBBrowserSourceClass parent;
 } RBLibrarySourceClass;
 
-GType		rb_library_source_get_type	(void);
+GType		rb_library_source_get_type		(void);
 
-RBSource *      rb_library_source_new           (RBShell *shell);
+RBSource *      rb_library_source_new			(RBShell *shell);
 
-void		rb_library_source_add_location	(RBLibrarySource *source, GtkWindow *win);
+void		rb_library_source_add_location		(RBLibrarySource *source, GtkWindow *win);
 
-void            rb_library_source_class_add_actions (RBShell *shell, 
-						     GtkActionGroup *uimgr);
-
-const char *	rb_library_source_get_paned_key	(RBLibrarySource *source);
 
 G_END_DECLS
 
