@@ -27,15 +27,19 @@
 #include <gtk/gtkactiongroup.h>
 
 #include "rb-entry-view.h"
+#include "rb-shell-preferences.h"
 
 G_BEGIN_DECLS
 
 typedef enum {
 	RB_SOURCE_EOF_ERROR,
+	RB_SOURCE_EOF_STOP,
 	RB_SOURCE_EOF_RETRY,
 	RB_SOURCE_EOF_NEXT,
 } RBSourceEOFType;
 
+GType rb_source_eof_type_get_type (void);
+#define RB_TYPE_SOURCE_EOF_TYPE	(rb_source_eof_type_get_type())
 
 #define RB_TYPE_SOURCE         (rb_source_get_type ())
 #define RB_SOURCE(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), RB_TYPE_SOURCE, RBSource))
@@ -54,8 +58,6 @@ typedef const char * (*RBSourceStringFunc) (RBSource *source);
 struct _RBSource
 {
 	GtkHBox parent;
-
-	RBSourcePrivate *priv;
 };
 
 struct _RBSourceClass
@@ -71,28 +73,34 @@ struct _RBSourceClass
 	/* methods */
 	char *	        (*impl_get_status)	(RBSource *source);
 
-	const char *	(*impl_get_browser_key)	(RBSource *source);
+	gboolean	(*impl_can_browse)	(RBSource *source);
+	char *		(*impl_get_browser_key)	(RBSource *source);
+	void		(*impl_browser_toggled)	(RBSource *source, gboolean enabled);
 
 	RBEntryView *	(*impl_get_entry_view)	(RBSource *source);
-	GList *		(*impl_get_extra_views)	(RBSource *source);
+	GList *		(*impl_get_property_views)	(RBSource *source);
 
-	GdkPixbuf *	(*impl_get_pixbuf)	(RBSource *source);
 	gboolean	(*impl_can_rename)	(RBSource *source);
 
 	gboolean	(*impl_can_search)	(RBSource *source);
 
 	void		(*impl_search)		(RBSource *source, const char *text);
 	void		(*impl_reset_filters)	(RBSource *source);
-	GtkWidget *	(*impl_get_config_widget)(RBSource *source);
+	GtkWidget *	(*impl_get_config_widget)(RBSource *source, RBShellPreferences *prefs);
 
 	gboolean	(*impl_can_cut)		(RBSource *source);
 	gboolean	(*impl_can_delete)	(RBSource *source);
+	gboolean	(*impl_can_move_to_trash) (RBSource *source);
 	gboolean	(*impl_can_copy)	(RBSource *source);
+	gboolean	(*impl_can_paste)	(RBSource *source);
+	gboolean	(*impl_can_add_to_queue)(RBSource *source);
 	
 	GList *		(*impl_cut)		(RBSource *source);
 	GList *		(*impl_copy)		(RBSource *source);
 	void		(*impl_paste)		(RBSource *source, GList *entries);
 	void		(*impl_delete)		(RBSource *source);
+	void		(*impl_add_to_queue)	(RBSource *source, RBSource *queue);
+	void		(*impl_move_to_trash)	(RBSource *source);
 
 	void		(*impl_song_properties)	(RBSource *source);
 
@@ -109,6 +117,8 @@ struct _RBSourceClass
 	void		(*impl_activate)	(RBSource *source);
 	void		(*impl_deactivate)	(RBSource *source);
 	gboolean	(*impl_disconnect)	(RBSource *source);
+	GList *		(*impl_get_ui_actions)	(RBSource *source);
+	GList *		(*impl_get_search_actions) (RBSource *source);
 };
 
 GType		rb_source_get_type		(void);
@@ -121,34 +131,40 @@ void		rb_source_update_play_statistics(RBSource *source, RhythmDB *db,
 						 RhythmDBEntry *entry);
 
 /* general interface */
+void	        rb_source_set_pixbuf		(RBSource *source, GdkPixbuf *pixbuf);
 char *	        rb_source_get_status		(RBSource *source);
 
-const char *	rb_source_get_browser_key	(RBSource *source);
+gboolean	rb_source_can_browse		(RBSource *source);
+char *		rb_source_get_browser_key	(RBSource *source);
+void		rb_source_browser_toggled	(RBSource *source, gboolean enabled);
 
 RBEntryView *	rb_source_get_entry_view	(RBSource *source);
 
-GList *		rb_source_get_extra_views	(RBSource *source);
+GList *		rb_source_get_property_views	(RBSource *source);
 
-GdkPixbuf *	rb_source_get_pixbuf		(RBSource *source);
 gboolean	rb_source_can_rename		(RBSource *source);
 
 gboolean	rb_source_can_search		(RBSource *source);
-
 void		rb_source_search		(RBSource *source,
 						 const char *text);
 
 void		rb_source_reset_filters		(RBSource *source);
 
-GtkWidget *	rb_source_get_config_widget	(RBSource *source);
+GtkWidget *	rb_source_get_config_widget	(RBSource *source, RBShellPreferences *prefs);
 
 gboolean	rb_source_can_cut		(RBSource *source);
 gboolean	rb_source_can_delete		(RBSource *source);
+gboolean	rb_source_can_move_to_trash	(RBSource *source);
 gboolean	rb_source_can_copy		(RBSource *source);
+gboolean	rb_source_can_paste		(RBSource *source);
+gboolean	rb_source_can_add_to_queue	(RBSource *source);
 
 GList *		rb_source_cut			(RBSource *source);
 GList *		rb_source_copy			(RBSource *source);
 void		rb_source_paste			(RBSource *source, GList *entries);
 void		rb_source_delete		(RBSource *source);
+void		rb_source_add_to_queue		(RBSource *source, RBSource *queue);
+void		rb_source_move_to_trash		(RBSource *source);
 
 void		rb_source_song_properties	(RBSource *source);
 
@@ -169,9 +185,21 @@ void		rb_source_activate		(RBSource *source);
 void		rb_source_deactivate		(RBSource *source);
 gboolean	rb_source_disconnect		(RBSource *source);
 
-/* Protected method, should only be used by objects inheriting from RBSource */
+GList *		rb_source_get_ui_actions	(RBSource *source);
+GList *		rb_source_get_search_actions	(RBSource *source);
+
+GList *		rb_source_gather_selected_properties (RBSource *source, RhythmDBPropType prop);
+
+/* Protected methods, should only be used by objects inheriting from RBSource */
 void            _rb_source_show_popup           (RBSource *source, 
 						 const char *ui_path);
+GtkActionGroup *_rb_source_register_action_group (RBSource *source,
+						  const char *group_name,
+						  GtkActionEntry *actions,
+						  int num_actions,
+						  gpointer user_data);
+void		_rb_source_hide_when_empty	(RBSource *source,
+						 RhythmDBQueryModel *model);
 
 G_END_DECLS
 
