@@ -1,5 +1,5 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
  *  Copyright (C) 2003 Colin Walters <walters@gnome.org>
  *
@@ -19,14 +19,17 @@
  *
  */
 
-#include <config.h>
-#include <gtk/gtk.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
-#include <libxml/tree.h>
+#include "config.h"
+
 #include <string.h>
 
+#include <libxml/tree.h>
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
+#include <libgnomevfs/gnome-vfs-uri.h>
+
 #include "rb-static-playlist-source.h"
+#include "rb-library-browser.h"
 #include "rb-util.h"
 #include "rb-debug.h"
 #include "rb-stock-icons.h"
@@ -35,21 +38,33 @@
 
 static GObject *rb_static_playlist_source_constructor (GType type, guint n_construct_properties,
 						       GObjectConstructParam *construct_properties);
+static void rb_static_playlist_source_dispose (GObject *object);
 static void rb_static_playlist_source_finalize (GObject *object);
 
 /* source methods */
-static GdkPixbuf *impl_get_pixbuf (RBSource *source);
 static GList * impl_cut (RBSource *source);
 static void impl_paste (RBSource *asource, GList *entries);
 static void impl_delete (RBSource *source);
 static void impl_search (RBSource *asource, const char *search_text);
+static void impl_browser_toggled (RBSource *source, gboolean enabled);
 static void impl_reset_filters (RBSource *asource);
 static gboolean impl_receive_drag (RBSource *source, GtkSelectionData *data);
+static GList *impl_get_search_actions (RBSource *source);
+static GPtrArray *construct_query_from_selection (RBStaticPlaylistSource *source);
 
 /* playlist methods */
 static void impl_save_contents_to_xml (RBPlaylistSource *source,
 				       xmlNodePtr node);
 
+
+/* browser stuff */
+static GList *impl_get_property_views (RBSource *source);
+void rb_static_playlist_source_browser_views_activated_cb (GtkWidget *widget,
+							 RBStaticPlaylistSource *source);
+static void rb_static_playlist_source_browser_changed_cb (RBLibraryBrowser *entry,
+							  GParamSpec *pspec,
+							  RBStaticPlaylistSource *source);
+
 static void rb_static_playlist_source_do_query (RBStaticPlaylistSource *source);
 
 static void rb_static_playlist_source_add_list_uri (RBStaticPlaylistSource *source,
@@ -63,6 +78,18 @@ static void rb_static_playlist_source_non_entry_dropped (GtkTreeModel *model,
 							 int position,
 							 RBStaticPlaylistSource *source);
 
+static void search_action_changed (GtkRadioAction *action,
+				   GtkRadioAction *current,
+				   RBShell *shell);
+
+static GtkRadioActionEntry rb_static_playlist_source_radio_actions [] =
+{
+	{ "StaticPlaylistSearchAll", NULL, N_("All"), NULL, N_("Search all fields"), 0 },
+	{ "StaticPlaylistSearchArtists", NULL, N_("Artists"), NULL, N_("Search artists"), 1 },
+	{ "StaticPlaylistSearchAlbums", NULL, N_("Albums"), NULL, N_("Search albums"), 2 },
+	{ "StaticPlaylistSearchTitles", NULL, N_("Titles"), NULL, N_("Search titles"), 3 }
+};
+
 G_DEFINE_TYPE (RBStaticPlaylistSource, rb_static_playlist_source, RB_TYPE_PLAYLIST_SOURCE)
 #define RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
 								RB_TYPE_STATIC_PLAYLIST_SOURCE, \
@@ -74,7 +101,14 @@ typedef struct
 	RhythmDBQueryModel *base_model;
 	RhythmDBQueryModel *filter_model;
 
+	GtkWidget *paned;
+	RBLibraryBrowser *browser;
+	gboolean browser_shown;
+
 	char *search_text;
+
+	GtkActionGroup *action_group;
+	RhythmDBPropType search_prop;
 } RBStaticPlaylistSourcePrivate;
 
 
@@ -88,19 +122,23 @@ rb_static_playlist_source_class_init (RBStaticPlaylistSourceClass *klass)
 	RBPlaylistSourceClass *playlist_class = RB_PLAYLIST_SOURCE_CLASS (klass);
 	
 	object_class->constructor = rb_static_playlist_source_constructor;
+	object_class->dispose = rb_static_playlist_source_dispose;
 	object_class->finalize = rb_static_playlist_source_finalize;
 
 	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_true_function;
-	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_true_function;
+	source_class->impl_can_paste = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_cut = impl_cut;
 	source_class->impl_paste = impl_paste;
 	source_class->impl_delete = impl_delete;
-	source_class->impl_get_pixbuf = impl_get_pixbuf;
 	source_class->impl_receive_drag = impl_receive_drag;
 	source_class->impl_can_search = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_search = impl_search;
 	source_class->impl_reset_filters = impl_reset_filters;
+	source_class->impl_can_browse = (RBSourceFeatureFunc) rb_true_function;
+	source_class->impl_browser_toggled = impl_browser_toggled;
+	source_class->impl_get_property_views = impl_get_property_views;
+	source_class->impl_get_search_actions = impl_get_search_actions;
 
 	playlist_class->impl_save_contents_to_xml = impl_save_contents_to_xml;
 	
@@ -116,7 +154,24 @@ rb_static_playlist_source_class_init (RBStaticPlaylistSourceClass *klass)
 static void
 rb_static_playlist_source_init (RBStaticPlaylistSource *source)
 {
-	
+	RBStaticPlaylistSourceClass *klass = RB_STATIC_PLAYLIST_SOURCE_GET_CLASS (source);
+
+	rb_source_set_pixbuf (RB_SOURCE (source), klass->pixbuf);
+}
+
+static void
+rb_static_playlist_source_dispose (GObject *object)
+{
+	RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (object);
+	if (!priv)
+		return;
+
+	if (priv->base_model) {
+		g_object_unref (G_OBJECT (priv->base_model));
+		priv->base_model = NULL;
+	}
+
+	G_OBJECT_CLASS (rb_static_playlist_source_parent_class)->dispose (object);
 }
 
 static void
@@ -138,11 +193,44 @@ rb_static_playlist_source_constructor (GType type, guint n_construct_properties,
 			parent_class->constructor (type, n_construct_properties, construct_properties));
 	RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (source);
 	RBPlaylistSource *psource = RB_PLAYLIST_SOURCE (source);
+	RBEntryView *songs;
+	RBShell *shell;
 
 	priv->base_model = rb_playlist_source_get_query_model (RB_PLAYLIST_SOURCE (psource));
 	g_object_set (G_OBJECT (priv->base_model), "show-hidden", TRUE, NULL);
-	
+	g_object_ref (G_OBJECT (priv->base_model));
+
+	priv->paned = gtk_vpaned_new ();
+
+	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
+	priv->action_group = _rb_source_register_action_group (RB_SOURCE (source),
+							       "StaticPlaylistActions",
+							       NULL, 0,
+							       shell);
+	gtk_action_group_add_radio_actions (priv->action_group,
+					    rb_static_playlist_source_radio_actions,
+					    G_N_ELEMENTS (rb_static_playlist_source_radio_actions),
+					    0,
+					    (GCallback)search_action_changed,
+					    shell);
+	priv->search_prop = RHYTHMDB_PROP_SEARCH_MATCH;
+	g_object_unref (G_OBJECT (shell));
+
+	priv->browser = rb_library_browser_new (rb_playlist_source_get_db (RB_PLAYLIST_SOURCE (source)));
+	gtk_paned_pack1 (GTK_PANED (priv->paned), GTK_WIDGET (priv->browser), TRUE, FALSE);
+	g_signal_connect_object (G_OBJECT (priv->browser), "notify::output-model",
+				 G_CALLBACK (rb_static_playlist_source_browser_changed_cb),
+				 source, 0);
+
+	rb_library_browser_set_model (priv->browser, priv->base_model, FALSE);
 	rb_static_playlist_source_do_query (source);
+			 
+	/* reparent the entry view */
+	songs = rb_source_get_entry_view (RB_SOURCE (source));
+	g_object_ref (G_OBJECT (songs));
+	gtk_container_remove (GTK_CONTAINER (source), GTK_WIDGET (songs));
+	gtk_paned_pack2 (GTK_PANED (priv->paned), GTK_WIDGET (songs), TRUE, FALSE);
+	gtk_container_add (GTK_CONTAINER (source), priv->paned);
 
 	/* watch these to find out when things are dropped into the entry view */
 	g_signal_connect_object (G_OBJECT (priv->base_model), "row-inserted",
@@ -152,6 +240,8 @@ rb_static_playlist_source_constructor (GType type, guint n_construct_properties,
 			 G_CALLBACK (rb_static_playlist_source_non_entry_dropped),
 			 source, 0);
 
+	gtk_widget_show_all (GTK_WIDGET (source));
+
 	return G_OBJECT (source);
 }
 
@@ -204,14 +294,6 @@ rb_static_playlist_source_new_from_xml (RBShell *shell, xmlNodePtr node)
 	return RB_SOURCE (source);
 }
 
-static GdkPixbuf *
-impl_get_pixbuf (RBSource *asource)
-{
-	RBStaticPlaylistSourceClass *klass = RB_STATIC_PLAYLIST_SOURCE_GET_CLASS (asource);
-	return klass->pixbuf;
-}
-
-
 static GList *
 impl_cut (RBSource *asource)
 {
@@ -254,6 +336,9 @@ impl_reset_filters (RBSource *source)
 {
 	RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (source);
 	gboolean changed = FALSE;
+	
+	if (rb_library_browser_reset (priv->browser))
+		changed = TRUE;
 
 	if (priv->search_text != NULL) {
 		changed = TRUE;
@@ -271,23 +356,72 @@ static void
 impl_search (RBSource *source, const char *search_text)
 {
 	RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (source);
+	char *old_search_text = NULL;
+
+	if (search_text != NULL && search_text[0] == '\0')
+		search_text = NULL;
 
 	if (search_text == NULL && priv->search_text == NULL)
 		return;
-	if (search_text != NULL && priv->search_text != NULL &&
-	    !strcmp (search_text, priv->search_text))
+	if (search_text != NULL && priv->search_text != NULL
+	    && !strcmp (search_text, priv->search_text))
 		return;
 
-	if (search_text[0] == '\0')
-		search_text = NULL;
+	old_search_text = priv->search_text;
+	if (search_text == NULL) {
+		priv->search_text = NULL;
+	} else {
+		priv->search_text = g_strdup (search_text);
+	}
+	g_free (old_search_text);
+	
+	rb_debug ("doing search for \"%s\"", priv->search_text ? priv->search_text : "(NULL)");
 
-	rb_debug ("doing search for \"%s\"", search_text ? search_text : "(NULL)");
-
-	g_free (priv->search_text);
-	priv->search_text = g_strdup (search_text);
 	rb_static_playlist_source_do_query (RB_STATIC_PLAYLIST_SOURCE (source));
+}
 
-	rb_source_notify_filter_changed (source);
+static GList *
+impl_get_property_views (RBSource *source)
+{
+	RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (source);
+	GList *ret;
+
+	ret =  rb_library_browser_get_property_views (priv->browser);
+	return ret;
+}
+
+
+static void
+impl_browser_toggled (RBSource *source, gboolean enabled)
+{
+	RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (source);
+
+	priv->browser_shown = enabled;
+
+	if (enabled)
+		gtk_widget_show (GTK_WIDGET (priv->browser));
+	else
+		gtk_widget_hide (GTK_WIDGET (priv->browser));
+}
+
+static GPtrArray *
+construct_query_from_selection (RBStaticPlaylistSource *source)
+{
+	RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (source);
+	RBPlaylistSource *psource = RB_PLAYLIST_SOURCE (source);
+	RhythmDB *db = rb_playlist_source_get_db (RB_PLAYLIST_SOURCE (psource));
+	GPtrArray *query = NULL;
+
+	query = g_ptr_array_new();
+
+	if (priv->search_text) {
+		rhythmdb_query_append (db,
+				       query,
+				       RHYTHMDB_QUERY_PROP_LIKE, priv->search_prop, priv->search_text,
+				       RHYTHMDB_QUERY_END);
+	}
+
+	return query;
 }
 
 static void
@@ -296,28 +430,36 @@ rb_static_playlist_source_do_query (RBStaticPlaylistSource *source)
 	RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (source);
 	RBPlaylistSource *psource = RB_PLAYLIST_SOURCE (source);
 	RhythmDB *db = rb_playlist_source_get_db (psource);
-	GPtrArray *query = NULL;
+	GPtrArray *query;
 	
 	if (priv->filter_model)
 		g_object_unref (priv->filter_model);
 	priv->filter_model = rhythmdb_query_model_new_empty (db);
 	g_object_set (G_OBJECT (priv->filter_model), "base-model", priv->base_model, NULL);
 
-	query = g_ptr_array_new();
-
-	if (priv->search_text) {
-		rhythmdb_query_append (db,
-				       query,
-				       RHYTHMDB_QUERY_PROP_LIKE, RHYTHMDB_PROP_SEARCH_MATCH, priv->search_text,
-				       RHYTHMDB_QUERY_END);
-	}
-
+	query = construct_query_from_selection (source);
 	g_object_set (G_OBJECT (priv->filter_model), "query", query, NULL);
 	rhythmdb_query_free (query);
+
 	rhythmdb_query_model_reapply_query (priv->filter_model, TRUE);
-	rb_playlist_source_set_query_model (psource, priv->filter_model);
+	rb_library_browser_set_model (priv->browser, priv->filter_model, FALSE);
 }
 
+static void
+rb_static_playlist_source_browser_changed_cb (RBLibraryBrowser *browser, 
+					      GParamSpec *pspec,
+					      RBStaticPlaylistSource *source)
+{
+	RBEntryView *songs = rb_source_get_entry_view (RB_SOURCE (source));
+	RhythmDBQueryModel *query_model;
+	
+	g_object_get (G_OBJECT (browser), "output-model", &query_model, NULL);
+	rb_entry_view_set_model (songs, query_model);
+	rb_playlist_source_set_query_model (RB_PLAYLIST_SOURCE (source), query_model);
+	g_object_unref (G_OBJECT (query_model));
+	
+	rb_source_notify_filter_changed (RB_SOURCE (source));
+}
 
 static gboolean
 impl_receive_drag (RBSource *asource, GtkSelectionData *data)
@@ -353,10 +495,12 @@ impl_save_contents_to_xml (RBPlaylistSource *source,
 		xmlNodePtr child_node = xmlNewChild (node, NULL, RB_PLAYLIST_LOCATION, NULL);
 		RhythmDBEntry *entry;
 		xmlChar *encoded;
+		const char *location;
 
 		gtk_tree_model_get (GTK_TREE_MODEL (priv->base_model), &iter, 0, &entry, -1);
 
-		encoded = xmlEncodeEntitiesReentrant (NULL, BAD_CAST entry->location);
+		location = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
+		encoded = xmlEncodeEntitiesReentrant (NULL, BAD_CAST location);
 
 		xmlNodeSetContent (child_node, encoded);
 		g_free (encoded);
@@ -369,11 +513,15 @@ rb_static_playlist_source_add_list_uri (RBStaticPlaylistSource *source,
 {
 	GList *i, *uri_list = NULL;
 	RBPlaylistSource *psource = RB_PLAYLIST_SOURCE (source);
+	RhythmDBEntry *entry;
 
 	g_return_if_fail (list != NULL);
 
-	for (i = list; i != NULL; i = g_list_next (i))
-		uri_list = g_list_prepend (uri_list, gnome_vfs_uri_to_string ((const GnomeVFSURI *) i->data, 0));
+	for (i = list; i != NULL; i = g_list_next (i)) {
+		char *uri = gnome_vfs_uri_to_string ((const GnomeVFSURI *) i->data, 0);
+		uri_list = g_list_prepend (uri_list, rb_canonicalise_uri (uri));
+		g_free (uri);
+	}
 	uri_list = g_list_reverse (uri_list);
 
 	gnome_vfs_uri_list_free (list);
@@ -384,7 +532,10 @@ rb_static_playlist_source_add_list_uri (RBStaticPlaylistSource *source,
 	for (i = uri_list; i != NULL; i = i->next) {
 		char *uri = i->data;
 		if (uri != NULL) {
-			rhythmdb_add_uri (rb_playlist_source_get_db (psource), uri);
+			entry = rhythmdb_entry_lookup_by_location (rb_playlist_source_get_db (psource), uri);
+			if (entry == NULL)
+				rhythmdb_add_uri (rb_playlist_source_get_db (psource), uri);
+
 			rb_static_playlist_source_add_location (source, uri, -1);
 		}
 
@@ -496,14 +647,20 @@ rb_static_playlist_source_add_entry (RBStaticPlaylistSource *source,
 				     RhythmDBEntry *entry,
 				     gint index)
 {
-	rb_static_playlist_source_add_location_internal (source, entry->location, index);
+	const char *location;
+
+	location = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
+	rb_static_playlist_source_add_location_internal (source, location, index);
 }
 
 void
 rb_static_playlist_source_remove_entry (RBStaticPlaylistSource *source,
 					RhythmDBEntry *entry)
 {
-	rb_static_playlist_source_remove_location (source, entry->location);
+	const char *location;
+
+	location = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
+	rb_static_playlist_source_remove_location (source, location);
 }
 
 void
@@ -544,3 +701,62 @@ rb_static_playlist_source_row_inserted (GtkTreeModel *model,
 	rb_static_playlist_source_add_entry (source, entry, -1);
 }
 
+static GList *
+impl_get_search_actions (RBSource *source)
+{
+	GList *actions = NULL;
+
+	actions = g_list_prepend (actions, g_strdup ("StaticPlaylistSearchTitles"));
+	actions = g_list_prepend (actions, g_strdup ("StaticPlaylistSearchAlbums"));
+	actions = g_list_prepend (actions, g_strdup ("StaticPlaylistSearchArtists"));
+	actions = g_list_prepend (actions, g_strdup ("StaticPlaylistSearchAll"));
+
+	return actions;
+}
+
+static RhythmDBPropType
+search_action_to_prop (GtkAction *action)
+{
+	const char      *name;
+	RhythmDBPropType prop;
+
+	name = gtk_action_get_name (action);
+
+	if (name == NULL) {
+		prop = RHYTHMDB_PROP_SEARCH_MATCH;
+	} else if (strcmp (name, "StaticPlaylistSearchAll") == 0) {
+		prop = RHYTHMDB_PROP_SEARCH_MATCH;		
+	} else if (strcmp (name, "StaticPlaylistSearchArtists") == 0) {
+		prop = RHYTHMDB_PROP_ARTIST_FOLDED;
+	} else if (strcmp (name, "StaticPlaylistSearchAlbums") == 0) {
+		prop = RHYTHMDB_PROP_ALBUM_FOLDED;
+	} else if (strcmp (name, "StaticPlaylistSearchTitles") == 0) {
+		prop = RHYTHMDB_PROP_TITLE_FOLDED;
+	} else {
+		prop = RHYTHMDB_PROP_SEARCH_MATCH;
+	}
+
+	return prop;
+}
+
+static void
+search_action_changed (GtkRadioAction  *action,
+		       GtkRadioAction  *current,
+		       RBShell         *shell)
+{
+	RBStaticPlaylistSource *source;
+	RBStaticPlaylistSourcePrivate *priv;
+	gboolean active;
+
+	g_object_get (G_OBJECT (shell), "selected-source", &source, NULL);
+	priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (source);
+
+	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (current));
+
+	if (active) {
+		/* update query */
+		priv->search_prop = search_action_to_prop (GTK_ACTION (current));
+		rb_static_playlist_source_do_query (source);
+		rb_source_notify_filter_changed (RB_SOURCE (source));
+	}
+}
