@@ -483,16 +483,6 @@ egg_tray_icon_send_message (EggTrayIcon *icon,
 		     
 #ifdef HAVE_NOTIFY
   egg_tray_icon_notify (icon, timeout, _("Notification"), NULL, message);
-#else
-  if (!icon->bubble)
-    {
-      icon->bubble = egg_notification_bubble_new ();
-      egg_notification_bubble_attach (icon->bubble, GTK_WIDGET (icon));
-    }
-
-  egg_notification_bubble_set (icon->bubble, _("Notification"),
-			       NULL, message);
-  egg_notification_bubble_show (icon->bubble, timeout);
 #endif
 
   return 1;
@@ -514,11 +504,6 @@ egg_tray_icon_cancel_message (EggTrayIcon *icon,
     icon->notify->handle = NULL;
 #endif
   }
-#else
-  g_return_if_fail (id > 0);
-  g_return_if_fail (icon->bubble != NULL);
-
-  egg_notification_bubble_hide (icon->bubble);
 #endif
 }
 
@@ -530,14 +515,6 @@ egg_tray_icon_get_orientation (EggTrayIcon *icon)
   return icon->orientation;
 }
 
-#ifndef HAVE_NOTIFY
-static void 
-egg_tray_icon_hide_notify_cb (EggNotificationBubble *bubble, gpointer data)
-{
-  egg_tray_icon_cancel_message (EGG_TRAY_ICON (data), 1);
-}
-#endif
-
 void
 egg_tray_icon_notify (EggTrayIcon *icon,
 		      guint timeout,
@@ -551,6 +528,8 @@ egg_tray_icon_notify (EggTrayIcon *icon,
   GdkPixbuf *pixbuf;
   int x;
   int y;
+  char *esc_primary;
+  char *esc_secondary;
 
   if (!notify_is_initted ())
     if (!notify_init ("rhythmbox"))
@@ -561,10 +540,14 @@ egg_tray_icon_notify (EggTrayIcon *icon,
       notify_notification_close (icon->notify->handle, NULL);
     }
 
-  icon->notify->handle = notify_notification_new (primary,
-                                                  secondary,
+  esc_primary = g_markup_escape_text (primary, strlen (primary));
+  esc_secondary = g_markup_escape_text (secondary, strlen (secondary));
+  icon->notify->handle = notify_notification_new (esc_primary,
+                                                  esc_secondary,
                                                   NULL,
-                                                  NULL);
+                                                  GTK_WIDGET (icon));
+  g_free (esc_primary);
+  g_free (esc_secondary);
 
   notify_notification_set_timeout (icon->notify->handle, timeout);
 
@@ -615,7 +598,8 @@ egg_tray_icon_notify (EggTrayIcon *icon,
   NotifyIcon *icon_notify = NULL;
   NotifyHints *hints;
   char *fn;
-  
+  char *esc_primary;
+  char *esc_secondary;
   
   if (!notify_is_initted ())
     if (!notify_init ("rhythmbox"))
@@ -663,6 +647,8 @@ egg_tray_icon_notify (EggTrayIcon *icon,
       notify_close (icon->notify->handle);
     }
 	  
+  esc_primary = g_markup_escape_text (primary, strlen (primary));
+  esc_secondary = g_markup_escape_text (secondary, strlen (secondary));
   icon->notify->hints = hints;
   icon->notify->icon = icon_notify;
   icon->notify->handle = notify_send_notification (NULL, "transfer",
@@ -674,25 +660,9 @@ egg_tray_icon_notify (EggTrayIcon *icon,
 			    	   	           hints,
 			    	   	           NULL,
 			    	   	           0);  
+  g_free (esc_primary);
+  g_free (esc_secondary);
   return;
 #endif
-#else
-  gint x, y;
-  gdk_window_get_origin (GTK_WIDGET (icon)->window,
-			 &x, &y);
-  if (!icon->bubble)
-    {
-      icon->bubble = egg_notification_bubble_new ();
-      egg_notification_bubble_attach (icon->bubble, GTK_WIDGET (icon));
-      g_signal_connect_object (icon->bubble,
-			       "clicked",
-			       G_CALLBACK (egg_tray_icon_hide_notify_cb),
-			       icon, 0);
-    }
-  
-  egg_notification_bubble_set (icon->bubble, primary,
-			       msgicon, secondary);
-  egg_notification_bubble_show (icon->bubble, timeout);
-  return;
 #endif
 }
