@@ -52,15 +52,24 @@ typedef enum {
 	RB_ENTRY_VIEW_COL_PLAY_COUNT,
 	RB_ENTRY_VIEW_COL_YEAR,
 	RB_ENTRY_VIEW_COL_LAST_PLAYED,
-	RB_ENTRY_VIEW_COL_FIRST_SEEN
+	RB_ENTRY_VIEW_COL_FIRST_SEEN,
+	RB_ENTRY_VIEW_COL_LAST_SEEN,
+	RB_ENTRY_VIEW_COL_LOCATION,
+	RB_ENTRY_VIEW_COL_ERROR
 } RBEntryViewColumn;
 
+GType rb_entry_view_column_get_type (void);
+#define RB_TYPE_ENTRY_VIEW_COLUMN	(rb_entry_view_column_get_type())
+
 typedef enum {
 	RB_ENTRY_VIEW_NOT_PLAYING,
 	RB_ENTRY_VIEW_PLAYING,
 	RB_ENTRY_VIEW_PAUSED
 } RBEntryViewState;
 
+GType rb_entry_view_state_get_type (void);
+#define RB_TYPE_ENTRY_VIEW_STATE	(rb_entry_view_state_get_type())
+
 typedef struct RBEntryViewPrivate RBEntryViewPrivate;
 
 typedef struct
@@ -84,7 +93,7 @@ typedef struct
 	void (*selection_changed)       (RBEntryView *view);
 	void (*sort_order_changed)     (RBEntryView *view);
 
-	void (*show_popup)             (RBEntryView *view);
+	void (*show_popup)             (RBEntryView *view, gboolean over_entry);
 } RBEntryViewClass;
 
 GType		rb_entry_view_get_type			(void);
@@ -93,7 +102,8 @@ RBEntryView *	rb_entry_view_new			(RhythmDB *db, GObject *shell_player,
 							 const char *sort_key, gboolean drag_source, 
 							 gboolean drag_dest);
 
-void		rb_entry_view_append_column		(RBEntryView *view, RBEntryViewColumn coltype);
+void		rb_entry_view_append_column		(RBEntryView *view, 
+							 RBEntryViewColumn coltype, gboolean always_visible);
 
 void		rb_entry_view_append_column_custom	(RBEntryView *view, GtkTreeViewColumn *column,
 							 const char *title, const char *key,
@@ -109,6 +119,7 @@ void		rb_entry_view_set_state			(RBEntryView *view,
 							 RBEntryViewState state);
 
 gboolean	rb_entry_view_have_selection		(RBEntryView *view);
+gboolean	rb_entry_view_have_complete_selection	(RBEntryView *view);
 GList *		rb_entry_view_get_selected_entries	(RBEntryView *view);
 
 void		rb_entry_view_select_all		(RBEntryView *view);
