@@ -47,6 +47,7 @@ typedef struct
 	GtkHBoxClass parent;
 
 	void (*search) (RBSearchEntry *view, const char *text);
+	void (*activate) (RBSearchEntry *entry);
 } RBSearchEntryClass;
 
 GType		rb_search_entry_get_type (void);
@@ -60,6 +61,8 @@ void		rb_search_entry_set_text (RBSearchEntry *entry, const char *text);
 
 gboolean	rb_search_entry_searching(RBSearchEntry *entry);
 
+void		rb_search_entry_grab_focus (RBSearchEntry *entry);
+
 G_END_DECLS
 
 #endif /* __RB_SEARCH_ENTRY_H */
